//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
///////////////////////////////////////////////////////////////////////////////
// This adaptor implements the async functions based on its own synchronous
// functions for testing purposes only. Since there is no principal need
// to do so, we allow these to be 'switched off'.
#if !defined (SAGA_DEFAULT_ADVERT_ADAPTOR_NO_ASYNCS)

#include <map>
#include <vector>

#include <boost/assert.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <boost/tokenizer.hpp>

#include <saga/exception.hpp>
#include <saga/url.hpp>
#include <saga/adaptors/task.hpp>
#include <saga/file.hpp>

#include <impl/config.hpp>
#include <impl/engine/task_get_bulk_adaptor.hpp>
#include <impl/exception_list.hpp>

#include "config.hpp"
#include "advertdirectory.hpp"
#include "common_helpers.hpp"

///////////////////////////////////////////////////////////////////////////////
namespace advert
{

///////////////////////////////////////////////////////////////////////////////
// attribute functions
saga::task_type 
advertdirectory_cpi_impl::async_attribute_exists(bool& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_attribute_exists",
        shared_from_this(), &advertdirectory_cpi_impl::sync_attribute_exists,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_attribute_is_readonly(bool& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_attribute_is_readonly",
        shared_from_this(), &advertdirectory_cpi_impl::sync_attribute_is_readonly,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_attribute_is_writable(bool& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_attribute_is_writable",
        shared_from_this(), &advertdirectory_cpi_impl::sync_attribute_is_writable,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_attribute_is_vector(bool& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_attribute_is_vector",
        shared_from_this(), &advertdirectory_cpi_impl::sync_attribute_is_vector,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_attribute_is_extended(bool& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_attribute_is_extended",
        shared_from_this(), &advertdirectory_cpi_impl::sync_attribute_is_extended,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_get_attribute(std::string& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_attribute",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_attribute,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_set_attribute(saga::void_t& ret, std::string key, 
    std::string val)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_set_attribute",
        shared_from_this(), &advertdirectory_cpi_impl::sync_set_attribute,
        TR1::ref(ret), key, val); 
}

saga::task_type 
advertdirectory_cpi_impl::async_get_vector_attribute(std::vector<std::string>& ret, 
     std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_vector_attribute",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_vector_attribute,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_set_vector_attribute(saga::void_t& ret, 
    std::string key, std::vector<std::string> val)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_set_vector_attribute",
        shared_from_this(), &advertdirectory_cpi_impl::sync_set_vector_attribute,
        TR1::ref(ret), key, val); 
}

saga::task_type 
advertdirectory_cpi_impl::async_remove_attribute(saga::void_t& ret, std::string key)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_remove_attribute",
        shared_from_this(), &advertdirectory_cpi_impl::sync_remove_attribute,
        TR1::ref(ret), key); 
}

saga::task_type 
advertdirectory_cpi_impl::async_list_attributes(std::vector<std::string>& keys)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_list_attributes",
        shared_from_this(), &advertdirectory_cpi_impl::sync_list_attributes,
        TR1::ref(keys)); 
}

saga::task_type 
advertdirectory_cpi_impl::async_find_attributes(std::vector<std::string>& keys,   
    std::string pattern)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_find_attributes",
        shared_from_this(), &advertdirectory_cpi_impl::sync_find_attributes,
        TR1::ref(keys), pattern); 
}

///////////////////////////////////////////////////////////////////////////////
// namespace_entry functions
saga::task_type 
advertdirectory_cpi_impl::async_get_url(std::string& url)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_url",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_url,
        TR1::ref(url)); 
}

saga::task_type 
advertdirectory_cpi_impl::async_get_cwd(std::string& url)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_cwd",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_cwd,
        TR1::ref(url)); 
}

saga::task_type 
advertdirectory_cpi_impl::async_get_name(std::string& url)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_name",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_name,
        TR1::ref(url)); 
}

saga::task_type advertdirectory_cpi_impl::async_is_dir(bool& ret, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_is_dir",
        shared_from_this(), &advertdirectory_cpi_impl::sync_is_dir,
        TR1::ref(ret), flags); 
}

saga::task_type advertdirectory_cpi_impl::async_is_entry(bool& ret, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_is_entry",
        shared_from_this(), &advertdirectory_cpi_impl::sync_is_entry,
        TR1::ref(ret), flags); 
}

saga::task_type advertdirectory_cpi_impl::async_is_link(bool& ret, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_is_link",
        shared_from_this(), &advertdirectory_cpi_impl::sync_is_link,
        TR1::ref(ret), flags); 
}

saga::task_type advertdirectory_cpi_impl::async_remove(saga::void_t& ret, 
    int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_remove",
        shared_from_this(), &advertdirectory_cpi_impl::sync_remove,
        TR1::ref(ret), flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_close(saga::void_t& ret, double timeout)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_close",
        shared_from_this(), &advertdirectory_cpi_impl::sync_close,
        TR1::ref(ret), timeout); 
}

///////////////////////////////////////////////////////////////////////////////
//  namespace_dir functions
saga::task_type 
advertdirectory_cpi_impl::async_list(std::vector<std::string>& ret, 
    std::string pattern, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_list",
        shared_from_this(), &advertdirectory_cpi_impl::sync_list,
        TR1::ref(ret), pattern, flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_find(std::vector<std::string>& ret, 
    std::string pattern, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_find",
        shared_from_this(), &advertdirectory_cpi_impl::sync_find,
        TR1::ref(ret), pattern, flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_is_link(bool& ret, std::string entry, 
    int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_is_link",
        shared_from_this(), &advertdirectory_cpi_impl::sync_is_link,
        TR1::ref(ret), entry, flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_exists(bool& ret, std::string entry)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_exists",
        shared_from_this(), &advertdirectory_cpi_impl::sync_exists,
        TR1::ref(ret), entry); 
}

saga::task_type 
advertdirectory_cpi_impl::async_is_dir(bool& ret, std::string entry, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_is_dir",
        shared_from_this(), &advertdirectory_cpi_impl::sync_is_dir,
        TR1::ref(ret), entry, flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_is_entry(bool& ret, std::string entry, int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_is_entry",
        shared_from_this(), &advertdirectory_cpi_impl::sync_is_entry,
        TR1::ref(ret), entry, flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_get_num_entries(unsigned int& ret)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_num_entries",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_num_entries,
        TR1::ref(ret)); 
}

saga::task_type 
advertdirectory_cpi_impl::async_get_entry(std::string& ret, unsigned int num)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_get_entry",
        shared_from_this(), &advertdirectory_cpi_impl::sync_get_entry,
        TR1::ref(ret), num); 
}

saga::task_type 
advertdirectory_cpi_impl::async_remove(saga::void_t& ret, std::string entry, 
    int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_remove",
        shared_from_this(), &advertdirectory_cpi_impl::sync_remove,
        TR1::ref(ret), entry, flags); 
}

saga::task_type 
advertdirectory_cpi_impl::async_change_dir(saga::void_t& ret, std::string dir)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_change_dir",
        shared_from_this(), &advertdirectory_cpi_impl::sync_change_dir,
        TR1::ref(ret), dir); 
}

saga::task_type 
advertdirectory_cpi_impl::async_make_dir(saga::void_t& ret, std::string dir, 
    int flags)
{
    return saga::adaptors::task("advertdirectory_cpi_impl::async_make_dir",
        shared_from_this(), &advertdirectory_cpi_impl::sync_make_dir,
        TR1::ref(ret), dir, flags); 
}

///////////////////////////////////////////////////////////////////////////////
// directory functions
saga::task_type 
advertdirectory_cpi_impl::async_open(saga::name_space::entry& ret, 
    std::string entry, int flags)
{
    typedef void (advertdirectory_cpi_impl::*sync_open_type)(
        saga::name_space::entry& ret, std::string entry, int flags);

    sync_open_type f = (sync_open_type)&advertdirectory_cpi_impl::sync_open;
    return saga::adaptors::task("advertdirectory_cpi_impl::async_open", 
        shared_from_this(), f, TR1::ref(ret), entry, flags);
}

saga::task_type 
advertdirectory_cpi_impl::async_open_dir(saga::name_space::directory& ret, 
    std::string entry, int flags)
{
    typedef void (advertdirectory_cpi_impl::*sync_open_dir_type)(
        saga::name_space::directory& ret, std::string entry, int flags);
        
    sync_open_dir_type f = (sync_open_dir_type)&advertdirectory_cpi_impl::sync_open_dir;
    return saga::adaptors::task("advertdirectory_cpi_impl::async_open_dir", 
        shared_from_this(), f, TR1::ref(ret), entry, flags);
}

saga::task_type 
advertdirectory_cpi_impl::async_open(saga::advert::entry& ret, 
    std::string entry, int flags)
{
    typedef void (advertdirectory_cpi_impl::*sync_open_type)(
        saga::advert::entry& ret, std::string entry, int flags);
        
    sync_open_type f = (sync_open_type)&advertdirectory_cpi_impl::sync_open;
    return saga::adaptors::task("advertdirectory_cpi_impl::async_open", 
        shared_from_this(), f, TR1::ref(ret), entry, flags);
}

saga::task_type 
advertdirectory_cpi_impl::async_open_dir(saga::advert::directory& ret, 
    std::string entry, int flags)
{
    typedef void (advertdirectory_cpi_impl::*sync_open_dir_type)(
        saga::advert::directory& ret, std::string entry, int flags);
        
    sync_open_dir_type f = (sync_open_dir_type)&advertdirectory_cpi_impl::sync_open_dir;
    return saga::adaptors::task("advertdirectory_cpi_impl::async_open_dir", 
        shared_from_this(), f, TR1::ref(ret), entry, flags);
}

saga::task_type 
advertdirectory_cpi_impl::async_find(std::vector<std::string>& ret, 
    std::string pattern, std::vector<std::string> patterns, int flags)
{
    typedef void (advertdirectory_cpi_impl::*sync_find_type)(
        std::vector<std::string>&, std::string, 
        std::vector<std::string>, int);

    sync_find_type f = (sync_find_type)&advertdirectory_cpi_impl::sync_find;
    return saga::adaptors::task("advertdirectory_cpi_impl::async_find",
        shared_from_this(), f, TR1::ref(ret), pattern, patterns, flags); 
}

///////////////////////////////////////////////////////////////////////////////
}   // namespace advert

#endif // SAGA_DEFAULT_ADVERT_ADAPTOR_NO_ASYNCS

