//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <saga/util.hpp>
#include <saga/adaptors/config.hpp>
#include <saga/adaptors/adaptor.hpp>
#include <saga/adaptors/task.hpp>

#include "default_file_adaptor.hpp"
#include "default_dir.hpp"
#include "default_file.hpp"
#if defined(SAGA_DEFAULT_FILE_ADAPTOR_USE_CURL)
#include "curl_file.hpp"
#include "curl_dir.hpp"
#endif

SAGA_ADAPTOR_REGISTER (file_adaptor);

//  register function for the SAGA engine
saga::impl::adaptor_selector::adaptor_info_list_type
    file_adaptor::adaptor_register(saga::impl::session *s)
{
    // list of descriptions for implemented cpi's
    saga::impl::adaptor_selector::adaptor_info_list_type infos;

    // create preferences
    preference_type prefs; // (std::string ("security"), std::string ("none"));

    // create file and directory cpi infos (each adaptor instance gets its own 
    // uuid) and add cpi_infos to list
    file_cpi_impl::register_cpi(infos, prefs, adaptor_uuid_);
    dir_cpi_impl::register_cpi(infos, prefs, adaptor_uuid_);

#if defined(SAGA_DEFAULT_FILE_ADAPTOR_USE_CURL)
    curl_file_cpi_impl::register_cpi(infos, prefs, adaptor_uuid_);
    curl_dir_cpi_impl::register_cpi(infos, prefs, adaptor_uuid_);
#endif

    // and return list
    return infos;
}

