//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef ADAPTORS_DEFAULT_FILE_DEFAULT_NAME_SPACE_DIR_HPP
#define ADAPTORS_DEFAULT_FILE_DEFAULT_NAME_SPACE_DIR_HPP

#include <string>

#include <saga/util.hpp>
#include <saga/adaptors/adaptor.hpp>
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/namespace_dir_cpi_instance_data.hpp>

#include <impl/engine/proxy.hpp>
#include <impl/packages/file/directory_cpi.hpp>

#include <impl/engine/task_get_bulk_adaptor.hpp>

#include <saga/uuid.hpp>

#include "default_namespace_entry.hpp"

///////////////////////////////////////////////////////////////////////////////
template <typename Base>
class namespace_dir_cpi_impl 
:   public namespace_entry_cpi_impl<Base>
{
  private:
    typedef namespace_entry_cpi_impl<Base> base_cpi;

  protected:
    typedef typename base_cpi::instance_data instance_data;

  public:
    /*! constructor of the namespace_dir cpi */
    namespace_dir_cpi_impl  (saga::impl::proxy * p, 
                              saga::impl::v1_0::cpi_info const & info,
                              saga::ini::ini const & glob_ini, 
                              saga::ini::ini const & adap_ini,
                              TR1::shared_ptr<saga::adaptor> adaptor,
                              saga::impl::v1_0::cpi::flags flags);

    /*! destructor of the namespace_dir cpi */
    ~namespace_dir_cpi_impl (void);

    /* implementation of query methods */
    void sync_exists   (bool & exists,  std::string url);
    void sync_is_dir   (bool & is_dir,  std::string url, int flags);
    void sync_is_entry (bool & is_file, std::string url, int flags);
    void sync_is_link  (bool & is_link, std::string url, int flags);
    
    /* implementation of saga::directory::open and saga::directory::open_directory */
//     void sync_open     (saga::file      & entry, std::string name, int openmode); 
//     void sync_open_dir (saga::directory & entry, std::string name, int openmode); 

    void sync_change_dir (saga::void_t&, std::string new_dir);
    void sync_list(std::vector <std::string> & retval, std::string pattern, int);
    void sync_find(std::vector <std::string> & retval, std::string entry, int);
    
    void sync_copy(saga::void_t & ret, std::string source, 
                   std::string destination, int flags);
    void sync_move(saga::void_t & ret, std::string source, 
                   std::string destination, int flags);
    void sync_remove(saga::void_t & ret, std::string url, 
                     int flags);
    void sync_make_dir(saga::void_t & ret, std::string url, 
                       int open_mode);
    void sync_link(saga::void_t & ret, std::string source, 
                   std::string url, int open_mode);
    void sync_read_link(std::string & ret, std::string source);

#if ! defined (SAGA_DEFAULT_FILE_ADAPTOR_NO_ASYNCS)
    // This adaptor implements the async functions based on its own synchronous
    // functions for testing purposes only. SInce there is no principal need
    // to do so, we allow these to be 'switched off'.
//     saga::task_type async_open     (saga::file      & entry, std::string name, int openmode); 
//     saga::task_type async_open_dir (saga::directory & entry, std::string name, int openmode); 

    saga::task_type async_exists   (bool & exists,  std::string url);
    saga::task_type async_is_entry (bool & is_file, std::string url, int);
    saga::task_type async_is_dir   (bool & is_dir,  std::string url, int);
    saga::task_type async_is_link  (bool & is_link, std::string url, int);

    saga::task_type async_change_dir (saga::void_t & retval, std::string new_dir);
    saga::task_type async_list(std::vector <std::string> & retval, std::string, int);
    saga::task_type async_find(std::vector <std::string> & retval, std::string, int);

    saga::task_type async_copy(saga::void_t & ret, std::string source, 
                          std::string destination, int flags);
    saga::task_type async_move(saga::void_t & ret, std::string source, 
                          std::string destination, int flags);
    saga::task_type async_remove(saga::void_t & ret, std::string url, 
                            int flags); 
    saga::task_type async_make_dir(saga::void_t & ret, std::string url, 
                              int open_mode);  
    
    saga::task_type async_link(saga::void_t & ret, std::string source, 
                          std::string url, int open_mode);
    saga::task_type async_read_link(std::string & ret, std::string source); 
    
#if ! defined (SAGA_DEFAULT_FILE_ADAPTOR_NO_BULKS)
    // bulk ops
    bool prepare_copy(saga::void_t & ret, std::string source, 
                      std::string destination, int open_mode, 
                      saga::uuid task_uuid=NULL);
    std::vector<saga::uuid> bulk_copy(
                      std::vector<TR1::reference_wrapper <saga::void_t> >& rets,
                      std::vector<std::string>& sources, 
                      std::vector<std::string>& dests, 
                      std::vector<int>& modes, std::vector<saga::uuid>& uuids);

    bool prepare_move(saga::void_t & ret, std::string source, 
                      std::string destination, int open_mode,
                      saga::uuid task_uuid=NULL);
    std::vector<saga::uuid> bulk_move(
                      std::vector<TR1::reference_wrapper <saga::void_t> >& rets,
                      std::vector<std::string>& sources, 
                      std::vector<std::string>& dests, 
                      std::vector<int>& modes, std::vector<saga::uuid>& uuids);
    
    bool prepare_remove(saga::void_t & ret, std::string url, 
                        int open_mode, saga::uuid task_uuid=NULL);
    std::vector<saga::uuid> bulk_remove(
                    std::vector<TR1::reference_wrapper <saga::void_t> >& rets,
                    std::vector<std::string>& urls, 
                    std::vector<int>& modes, 
                    std::vector<saga::uuid>& uuids);

#endif // ! SAGA_DEFAULT_FILE_ADAPTOR_NO_BULKS
#endif // ! SAGA_DEFAULT_FILE_ADAPTOR_NO_ASYNCS

};  // class namespace_dir_cpi_impl

#endif // ADAPTORS_DEFAULT_FILE_DEFAULT_NAME_SPACE_DIR_HPP

