//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef ADAPTORS_DEFAULT_FILE_DEFAULT_NAME_SPACE_ENTRY_HPP
#define ADAPTORS_DEFAULT_FILE_DEFAULT_NAME_SPACE_ENTRY_HPP

#include <vector>
#include <string>
#include <fstream>

#include <saga/util.hpp>
#include <saga/adaptors/adaptor.hpp>
#include <saga/adaptors/instance_data.hpp>

#include <impl/engine/proxy.hpp>
#include <impl/packages/file/directory_cpi.hpp>

#include <impl/engine/task_get_bulk_adaptor.hpp>

///////////////////////////////////////////////////////////////////////////////
template <typename Base>
class namespace_entry_cpi_impl : public Base
{
  private:
    typedef Base base_cpi;
    
  protected:
    typedef typename Base::instance_data instance_data;
    
  public:
    /*! constructor of the namespace_entry adaptor */
    namespace_entry_cpi_impl  (saga::impl::proxy* p, 
                                saga::impl::v1_0::cpi_info const & info,
                                saga::ini::ini const & glob_ini, 
                                saga::ini::ini const & adap_ini,
                                TR1::shared_ptr<saga::adaptor> adaptor,
                                saga::impl::v1_0::cpi::flags flags);

    /*! destructor of the ns_entry adaptor */
    ~namespace_entry_cpi_impl (void);

    /* implementation of query methods */
    void sync_exists   (bool & exists);
    void sync_is_dir   (bool & is_dir, int flags);
    void sync_is_entry (bool & is_file, int flags);
    void sync_is_link  (bool & is_link, int flags);
    
    /*! implementation of saga::directory::get_url */
    void sync_get_url  (std::string & url);
    void sync_get_cwd  (std::string & url);
    void sync_get_name (std::string & url);

    void sync_copy (saga::void_t & ret, std::string source, int flags);
    void sync_link (saga::void_t & ret, std::string source, int flags);
    void sync_move (saga::void_t & ret, std::string source, int flags);
    void sync_remove (saga::void_t & ret, int flags);
    void sync_close (saga::void_t & ret, double);

#if ! defined (SAGA_DEFAULT_FILE_ADAPTOR_NO_ASYNCS)
    // This adaptor implements the async functions based on its own synchronous
    // functions for testing purposes only. SInce there is no principal need
    // to do so, we allow these to be 'switched off'.
    saga::task_type async_get_url  (std::string & url);
    saga::task_type async_get_cwd  (std::string & url);
    saga::task_type async_get_name (std::string & url);

    saga::task_type async_exists   (bool & exists);
    saga::task_type async_is_entry (bool & is_file, int flags);
    saga::task_type async_is_dir   (bool & is_dir, int flags);
    saga::task_type async_is_link  (bool & is_link, int flags);
    
    saga::task_type async_copy (saga::void_t & ret, std::string dest, int flags);
    saga::task_type async_link (saga::void_t & ret, std::string target, int flags);
    saga::task_type async_move (saga::void_t & ret, std::string dest, int flags);
    saga::task_type async_remove (saga::void_t & ret, int flags);
    saga::task_type async_close (saga::void_t & ret, double);

#if ! defined (SAGA_DEFAULT_FILE_ADAPTOR_NO_BULKS)
    // Bulk ops
    bool prepare_get_name(std::string& url, saga::uuid task_uuid);
    std::vector<saga::uuid> bulk_get_name(
                            std::vector<TR1::reference_wrapper<std::string> >&,
                            std::vector<saga::uuid > & params);
#endif // ! SAGA_DEFAULT_FILE_ADAPTOR_NO_BULKS
#endif // ! SAGA_DEFAULT_FILE_ADAPTOR_NO_ASYNCS

};  // class namespace_entry_cpi_impl

#endif // ADAPTORS_DEFAULT_FILE_DEFAULT_NAME_SPACE_ENTRY_HPP

