//  Copyright (c) 2006 Ole Christian Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <fstream>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>

#include <saga/adaptors/adaptor.hpp>

#include "default_test_helper_utils.hpp"

SAGA_TEST_HELPER_REGISTER(test_helper_utils_impl); 

///////////////////////////////////////////////////////////////////////////////
//
void test_helper_utils_impl::delete_temp_file (std::string path)
{
    mutex_type::scoped_lock lock(mtx_);
}
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
void test_helper_utils_impl::delete_temp_dir  (std::string path)
{
    mutex_type::scoped_lock lock(mtx_);
}
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
std::string 
test_helper_utils_impl::create_temp_file_for_exception( const saga::error & e )
{
  mutex_type::scoped_lock lock(mtx_);
  std::string unique_path = get_unique_path_name();
  saga::url unique_url;
  
  switch (e) {
    
     case(saga::Success):
     {
       unique_url = saga::url(unique_path);
       unique_url.set_scheme("file");
       unique_url.set_host("localhost");
       
       boost::filesystem::path p (unique_url.get_path(), 
                                boost::filesystem::native);
       
       std::fstream file;
       
       file.open (p.string().c_str (), std::ios::out);
       file.close();
     }
       break;
      
     case(saga::IncorrectState):
     {
       unique_url = saga::url(unique_path);
       unique_url.set_scheme("file");
       unique_url.set_host("localhost");
       
       boost::filesystem::path p (unique_url.get_path(), 
                                boost::filesystem::native);
       
       std::fstream file;
       
       file.open (p.string().c_str (), std::ios::out);
       file.close();
     }
       break;
      
     case(saga::BadParameter):
     {
       unique_url = saga::url(unique_path);
       unique_url.set_scheme("file");
       unique_url.set_host("localhost");
       
       boost::filesystem::path p (unique_url.get_path(), 
                                 boost::filesystem::native);
       
       std::fstream file;
       
       file.open (p.string().c_str (), std::ios::out);
       file.close();
     }
       break;
     case(saga::DoesNotExist):
     {
       unique_url = saga::url(unique_path);
       unique_url.set_scheme("file");
       unique_url.set_host("localhost");
     }
       break;
       
     case(saga::IncorrectURL):
     {
       unique_url = saga::url("file:local:-host/~.../");
     
     }
       break;
       
     case(saga::PermissionDenied):
     {
       // create a file with insufficent permissions and return the url
     }
       break;
       
     default:
     {
       unique_url = saga::url("");
     }
       break;
   }
  
  return unique_url.get_url();
}
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
std::string 
test_helper_utils_impl::create_temp_dir_for_exception( const saga::error & e )
{
  mutex_type::scoped_lock lock(mtx_);
  std::string unique_path = get_unique_path_name();
  saga::url unique_url;
  
  switch (e) {
    
     case(saga::Success):
     {
        std::string unique_path = get_unique_path_name();
        std::string::size_type pos = unique_path.find_first_of(SAGA_PATH_DELIMITERS);
        if (unique_path.size()-1 != pos)
            unique_path += "/";
        boost::filesystem::path p (unique_url.get_path());
        try {
           boost::filesystem::create_directory(unique_path);
//                           boost::filesystem::native);
        } 
        catch(boost::filesystem::filesystem_error const& e) {
           std::cerr << e.what() << std::endl;
        }
        unique_url = saga::url(unique_path);
        unique_url.set_scheme("file");
        unique_url.set_host("localhost"); 
     }
       break;
     //Just make a normal one and let tests take care of rest
     case(saga::AlreadyExists):
     {
        std::string unique_path = get_unique_path_name();
        std::string::size_type pos = unique_path.find_first_of(SAGA_PATH_DELIMITERS);
        if (unique_path.size()-1 != pos)
            unique_path += "/";
        boost::filesystem::path p (unique_url.get_path());
        try {
           boost::filesystem::create_directory(unique_path);
//                           boost::filesystem::native);
        } 
        catch(boost::filesystem::filesystem_error const& e) {
           std::cerr << e.what() << std::endl;
        }
        unique_url = saga::url(unique_path);
        unique_url.set_scheme("file");
        unique_url.set_host("localhost"); 
     }
       break;
      
     case(saga::IncorrectState):
     {
        std::string unique_path = get_unique_path_name();
        std::string::size_type pos = unique_path.find_first_of(SAGA_PATH_DELIMITERS);
        if (unique_path.size()-1 != pos)
            unique_path += "/";
        boost::filesystem::path p(unique_path);
//                           boost::filesystem::native);
        try {
           boost::filesystem::create_directory(p);
        } 
        catch(boost::filesystem::filesystem_error const& e) {
           std::cerr << e.what() << std::endl;
        }
        unique_url = saga::url(unique_path);
        unique_url.set_scheme("file");
        unique_url.set_host("localhost"); 
     }
       break;
      
     case(saga::BadParameter):
     {
        std::string unique_path = get_unique_path_name();
        std::string::size_type pos = unique_path.find_first_of(SAGA_PATH_DELIMITERS);
        if (unique_path.size()-1 != pos)
            unique_path += "/";
        boost::filesystem::path p(unique_path);
//                           boost::filesystem::native);
        try {
           boost::filesystem::create_directory(p);
        } 
        catch(boost::filesystem::filesystem_error const& e) {
           std::cerr << e.what() << std::endl;
        }
        unique_url = saga::url(unique_path);
        unique_url.set_scheme("file");
        unique_url.set_host("localhost"); 
     }
       break;
     case(saga::DoesNotExist):
     {
        std::string unique_path = get_unique_path_name();
        std::string::size_type pos = unique_path.find_first_of(SAGA_PATH_DELIMITERS);
        if (unique_path.size()-1 != pos)
            unique_path += "/";
        unique_url = saga::url(unique_path);
        unique_url.set_scheme("file");
        unique_url.set_host("localhost"); 
     }
       break;
       
     case(saga::IncorrectURL):
     {
       unique_url = saga::url("file:local:-host/~.../");
     
     }
       break;
       
     case(saga::PermissionDenied):
     {
       // create a file with insufficent permissions and return the url
     }
       break;
       
     default:
     {
       unique_url = std::string("");
     }
       break;
   }
//   return unique_path;
   return unique_url.get_url();
}
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
std::string test_helper_utils_impl::create_temp_file_name (bool create)
{
  mutex_type::scoped_lock lock(mtx_);
  std::string unique_path = get_unique_path_name();
  saga::url unique_url(unique_path);
  unique_url.set_scheme("file");
  unique_url.set_host("localhost");
  
  if(create) {
    boost::filesystem::path p (unique_url.get_path(), 
                               boost::filesystem::native);
    
    std::fstream file;
    
    file.open (p.string().c_str (), std::ios::out);
    file.close();
  }
  return unique_url.get_url();
}
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//
std::string test_helper_utils_impl::create_temp_dir_name (bool create)
{
  mutex_type::scoped_lock lock(mtx_);
  std::string unique_path = get_unique_path_name();
  std::string::size_type pos = unique_path.find_first_of(SAGA_PATH_DELIMITERS);
  if (unique_path.size()-1 != pos)
      unique_path += "/";
      
  saga::url unique_url(unique_path);
  unique_url.set_scheme("file");
  unique_url.set_host("localhost"); 
  if(create) {
    boost::filesystem::path p(unique_url.get_path(), 
                              boost::filesystem::native);
    
    try {
        boost::filesystem::create_directory(p);
    } 
    catch(boost::filesystem::filesystem_error const& e) {
        std::cerr << e.what() << std::endl;
    }
  }
  return unique_url.get_url();
}

