//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <map>
#include <vector>
#include <algorithm>

#include <boost/function_output_iterator.hpp>

#include <saga/adaptors/config.hpp>
#include <saga/adaptors/adaptor.hpp>
#include <saga/adaptors/task.hpp>

#include "default_job_adaptor.hpp"
#include "default_job_service.hpp"
#include "default_job.hpp"

SAGA_ADAPTOR_REGISTER (job_adaptor);

/*
 * register function for the SAGA engine
 */
saga::impl::adaptor_selector::adaptor_info_list_type
    job_adaptor::adaptor_register(saga::impl::session *s)
{
    // list of implemented cpi's
    saga::impl::adaptor_selector::adaptor_info_list_type list;

    // create preferences
    preference_type prefs; // (std::string ("security"), std::string ("none"));

    // create file adaptor infos (each adaptor instance gets its own uuid)
    // and add cpi_infos to list
    job_service_cpi_impl::register_cpi(list, prefs, adaptor_uuid_);
    job_cpi_impl::register_cpi(list, prefs, adaptor_uuid_);
                                    
    // and return list
    return (list);
}

///////////////////////////////////////////////////////////////////////////////
bool job_adaptor::register_job(std::string jobid, saga::job::description jd)
{
    std::pair<known_jobs_type::iterator, bool> p =
        known_jobs_.insert(known_jobs_type::value_type(jobid, jd));
    return p.second;
}

bool job_adaptor::unregister_job(std::string jobid)
{
    known_jobs_type::iterator it = known_jobs_.find(jobid);
    if (it == known_jobs_.end())
        return false;
        
    known_jobs_.erase(it);
    return true;
}

saga::job::description 
job_adaptor::get_job(job_cpi_impl const* job, std::string jobid) const
{
    known_jobs_type::const_iterator it = known_jobs_.find(jobid);
    if (it == known_jobs_.end()) {
        SAGA_ADAPTOR_THROW_VERBATIM(job, "Nothing known about job: " + jobid, 
            saga::BadParameter);
    }
    return (*it).second;
}

///////////////////////////////////////////////////////////////////////////////
namespace {

    struct string_appender
    {
        typedef job_adaptor::known_jobs_type::value_type value_type;
        
        string_appender(std::vector<std::string>& v) : v_(v) {}
        void operator()(value_type v) { v_.push_back(v.first); }
        std::vector<std::string>& v_;
    };
}

std::vector<std::string> job_adaptor::list_jobs() const
{
    std::vector <std::string> jobids;
    std::copy(known_jobs_.begin(), known_jobs_.end(),
          boost::make_function_output_iterator(string_appender(jobids)));
    return jobids;
}

bool job_adaptor::knows_job(std::string jobid) const
{
    return known_jobs_.find(jobid) != known_jobs_.end();
}
