//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <string>

#include <saga/base.hpp>
#include <saga/util.hpp>
#include <saga/task.hpp>

#include <impl/call.hpp>
#include <impl/engine/cpi_info.hpp>
#include <impl/engine/sync_async.hpp>
#include <impl/engine/permissions.hpp>
#include <impl/engine/proxy.hpp>
#include <impl/engine/cpi_info.hpp>
#include <impl/engine/cpi_list.hpp>
#include <impl/engine/run_mode_wrapper.hpp>

#include <saga/adaptors/task.hpp>
#include <saga/adaptors/permissions_cpi.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace impl
{
    ///////////////////////////////////////////////////////////////////////////
    saga::impl::proxy* permissions::get_proxy()
    {
        SAGA_THROW(
            "This object does not expose the SAGA proxy interface", 
            saga::NotImplemented);
        return NULL;
    }

    saga::impl::proxy const* permissions::get_proxy() const
    {
        SAGA_THROW(
            "This object does not expose the SAGA proxy interface", 
            saga::NotImplemented);
        return NULL;
    }

    ///////////////////////////////////////////////////////////////////////////
    saga::task_type 
    permissions::permissions_allow (saga::void_t& ret, 
        std::string id, int perm, bool is_sync)
    {
        preference_type prefs;
        return saga::impl::execute_sync_async(get_proxy(),
            "permissions_cpi", "permissions_allow", 
            "permissions::permissions_allow", prefs, is_sync,
            &permissions_cpi::sync_permissions_allow, 
            &permissions_cpi::async_permissions_allow, 
            TR1::ref(ret), id, perm);
    }

    saga::task_type 
    permissions::permissions_deny (saga::void_t& ret, 
        std::string id, int perm, bool is_sync)
    {
        preference_type prefs;
        return saga::impl::execute_sync_async(get_proxy(),
            "permissions_cpi", "permissions_deny", 
            "permissions::permissions_deny", prefs, is_sync,
            &permissions_cpi::sync_permissions_deny, 
            &permissions_cpi::async_permissions_deny, 
            TR1::ref(ret), id, perm);
    }

    saga::task_type 
    permissions::permissions_check (bool& ret, 
        std::string id, int perm, bool is_sync)
    {
        preference_type prefs;
        return saga::impl::execute_sync_async(get_proxy(),
            "permissions_cpi", "permissions_check", 
            "permissions::permissions_check", prefs, is_sync,
            &permissions_cpi::sync_permissions_check, 
            &permissions_cpi::async_permissions_check, 
            TR1::ref(ret), id, perm);
    }

    saga::task_type 
    permissions::get_owner (std::string& ret, bool is_sync)
    {
        preference_type prefs;
        return saga::impl::execute_sync_async(get_proxy(),
            "permissions_cpi", "get_owner", 
            "permissions::get_owner", prefs, is_sync,
            &permissions_cpi::sync_get_owner, 
            &permissions_cpi::async_get_owner, 
            TR1::ref(ret));
    }

    saga::task_type 
    permissions::get_group (std::string& ret, bool is_sync)
    {
        preference_type prefs;
        return saga::impl::execute_sync_async(get_proxy(),
            "permissions_cpi", "get_group", 
            "permissions::get_group", prefs, is_sync,
            &permissions_cpi::sync_get_group, 
            &permissions_cpi::async_get_group, 
            TR1::ref(ret));
    }

///////////////////////////////////////////////////////////////////////////////
}}

