//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005 Michel Zandstra (michelzandstra@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <boost/shared_ptr.hpp>

#include <impl/config.hpp>
#include <impl/session.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {

    // saga:: object interface: clone this session instance
    saga::object session::clone() const
    {
        mutex_type::scoped_lock lock(mtx_);

        saga::session s;
        context_list_type::const_iterator end = contexts_.end();
        for (context_list_type::const_iterator it = contexts_.begin();
             it != end; ++it)
        {
            s.add_context(*it);
        }
        return s;
    }
    
    // maintain internal list of context's
    std::list <saga::context> 
    session::list_contexts (saga::context::type t) const
    {
        mutex_type::scoped_lock lock(mtx_);
        if (t == saga::context::Any)
            return contexts_;
    
        std::list<saga::context> ctxs;
        std::list<saga::context>::const_iterator end = contexts_.end();
        for (std::list<saga::context>::const_iterator it = contexts_.begin(); 
             it != end; ++it)
        {
            if (t == (*it).get_ctype()) 
                ctxs.push_back(*it);
        }
        return ctxs;
    }
    
    void session::add_context (saga::context const& c)
    {
        mutex_type::scoped_lock lock(mtx_);
        context_list_type::iterator it = std::find(contexts_.begin(), contexts_.end(), c);
        if (it == contexts_.end())
            contexts_.push_back (c.clone());
    }
    
    void session::remove_context (saga::context const& c)
    {
        mutex_type::scoped_lock lock(mtx_);
        context_list_type::iterator it = std::find(contexts_.begin(), contexts_.end(), c);
        if (it == contexts_.end())
        {
            SAGA_THROW("remove_context: context does not exist", 
                saga::DoesNotExist);
        }
        contexts_.erase (it);
    }

  } // namespace impl

} // namespace saga

