//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <saga/base.hpp>
#include <saga/exception.hpp>
#include <impl/exception_list.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { 

///////////////////////////////////////////////////////////////////////////////
//  This is needed to find the most significant error in a exception list
/// @cond
/** These methods are not within API */
bool operator< (saga::exception const& lhs, saga::exception const& rhs) 
{
    return lhs.get_error() < rhs.get_error();
}
/// @endcond

///////////////////////////////////////////////////////////////////////////////
namespace impl {
    
    exception_list::exception_list()
    {
    }
    
    exception_list::exception_list(saga::exception const& e)
    {
        add(e);
    }
    
    void exception_list::add(saga::exception const& e)
    {
        exceptions_.push_back(e);
    }
    
    saga::error exception_list::get_error() const
    {
        if (exceptions_.empty())
            return saga::Success;

        // the min_element algorithm below uses the bool operator<(...) defined 
        // at the beginning of this file to compare the error codes of the
        // exceptions
        exception_list_type::const_iterator min = 
            std::min_element(exceptions_.begin(), exceptions_.end());
        return (*min).get_error();
    }

    namespace {
    
        std::string indent_message(char const* msg_) 
        {
            std::string result;
            std::string msg(msg_);
            std::string::size_type pos = msg.find_first_of("\n");
            std::string::size_type first_non_ws = msg.find_first_not_of(" \n");
            std::string::size_type pos1 = 0;
            
            while (std::string::npos != pos) {
                if (pos > first_non_ws) {   // skip leading newline
                    result += msg.substr(pos1, pos-pos1+1); 
                    pos = msg.find_first_of("\n", pos1 = pos+1);
                    if (std::string::npos != pos)
                        result += "  ";
                }
                else {
                    pos = msg.find_first_of("\n", pos1 = pos+1);
                }
            }
            
            result += msg.substr(pos1); 
            return result;
        }
    }
        
    std::string exception_list::get_message() const
    {
        if (exceptions_.empty())
            return "";

        std::string result("\n");

        exception_list_type::const_iterator end = exceptions_.end();
        exception_list_type::const_iterator it = exceptions_.begin();
        for (/**/; it != end; ++it) {
            result += "  ";
            result += indent_message((*it).what());
            if (result.find_last_of("\n") < result.size()-1)
                result += "\n";
        }
        return result;
    }

    std::size_t exception_list::get_error_count() const
    {
        return exceptions_.size();
    }
    
///////////////////////////////////////////////////////////////////////////////
}}   // namespace saga::impl
