#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/directory.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif

#include <saga/url.hpp>

#include <saga/adaptors/task.hpp>

// needs tasks for async ops
#include <impl/config.hpp>
#include <impl/call.hpp>
#include <impl/engine/task.hpp>
#include <impl/engine/sync_async.hpp>

// own header file
#include <impl/packages/advert/advert_directory.hpp>
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/advert_directory_cpi_instance_data.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/advert/preprocessed/directory.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: "preprocessed/directory.cpp")
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  namespace impl 
  {
    advert_directory::advert_directory (saga::session const& s, 
            std::string const& url, int mode) 
        : namespace_dir (saga::object::AdvertDirectory, s)
    {
        // initialize the instance data
        typedef adaptors::v1_0::advert_directory_cpi_instance_data 
            instance_data_type;
        typedef adaptors::instance_data <instance_data_type> 
            advert_dir_instance_data;

        // make sure, url is valid and directory path ends with a '/'
        saga::url dir_url(url);
        std::string pathstr (dir_url.get_path());
        std::string::size_type pos = pathstr.find_last_of(SAGA_PATH_DELIMITERS);
        if (pathstr.size() > 0 && pathstr.size()-1 != pos) 
        {
            pathstr += "/";
            dir_url.set_path(pathstr);
        }

        {
            advert_dir_instance_data data;
            data.init_data (this, TR1::shared_ptr <instance_data_type> (
                new instance_data_type (dir_url.get_url(), mode)));
        } // lock goes out of scope
    }
    
    advert_directory::advert_directory (attribute const& rhs_attr)
    :   saga::impl::namespace_dir(saga::object::AdvertDirectory),
        attribute_base(rhs_attr)
    {
    }
    
    void advert_directory::init()
    {
        // initialize a CPI instance and execute required functions (such as
        // creating the directory etc.)
        this->initcpi(saga::adaptors::advert_directory_cpi);
    }
    
    advert_directory::~advert_directory (void) 
    {
        // free the instance data
        typedef adaptors::v1_0::advert_directory_cpi_instance_data 
            instance_data_type;
        typedef adaptors::instance_data <instance_data_type> 
            logical_dir_instance_data;

        logical_dir_instance_data data;
        data.release_data (this);
    }

    // overload open methods of name space
    SAGA_CALL_IMPL_IMPL_2 (advert_directory, advert_directory_cpi, saga::advert::entry, open, std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (advert_directory, advert_directory_cpi, saga::advert::directory, open_dir, std::string, int)
    SAGA_CALL_IMPL_IMPL_3 (advert_directory, advert_directory_cpi, std::vector<std::string>, find, std::string, std::vector<std::string>, int)

    // Generate a exact deep copy of this object
    saga::object advert_directory::clone() const
    {
        return saga::advert::directory(
            new advert_directory(static_cast<attribute_base const&>(*this)));
    }

  } // namespace impl

///////////////////////////////////////////////////////////////////////////////
} // namespace saga

#endif

