#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/directory.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005 Michel Zandstra (michelzandstra@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(preserve: 0, output: null)
#endif
#include <saga/util.hpp>
#include <saga/url.hpp>

#include <saga/adaptors/task.hpp>

// needs tasks for async ops
#include <impl/call.hpp>
#include <impl/engine/task.hpp>
#include <impl/engine/sync_async.hpp>

// own header file
#include <impl/packages/file/directory.hpp>
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/directory_cpi_instance_data.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/file/preprocessed/directory.cpp>
#else

#if defined(__WAVE__)
#pragma wave option(output: "preprocessed/directory.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  namespace impl 
  {
    directory::directory (saga::session const & s, 
                          std::string   const & url, 
                          int                   mode) 
        : namespace_dir (saga::object::Directory, s)
    {
        // initialize the instance data
        typedef adaptors::v1_0::directory_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> dir_instance_data;

        // make sure, url is valid and directory path ends with a '/'
        saga::url dir_url(url);
        std::string pathstr (dir_url.get_path());
        std::string::size_type pos = pathstr.find_last_of(SAGA_PATH_DELIMITERS);
        if (pathstr.empty() || pathstr.size()-1 != pos) 
        {
          pathstr += "/";
          dir_url.set_path(pathstr);
        }

        {
            dir_instance_data data;
            data.init_data (this, TR1::shared_ptr <instance_data_type> (
                new instance_data_type (dir_url.get_url(), mode)));
        } // lock goes out of scope
    }
    
    void directory::init()
    {
        // initialize a CPI instance and execute required functions (such as
        // creating the directory etc.)
        this->initcpi(saga::adaptors::directory_cpi);
    }
    
    directory::~directory (void) 
    {
        // free the instance data
        typedef adaptors::v1_0::directory_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> dir_instance_data;

        dir_instance_data data;
        data.release_data (this);
    }

    // file info
    SAGA_CALL_IMPL_IMPL_1 (directory, directory_cpi, saga::off_t,     get_size, std::string)
    SAGA_CALL_IMPL_IMPL_1 (directory, directory_cpi, bool,            is_file,  std::string)

    // overload open methods of name space
    SAGA_CALL_IMPL_IMPL_2 (directory, directory_cpi, saga::filesystem::file,      open,     std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (directory, directory_cpi, saga::filesystem::directory, open_dir, std::string, int)

  } // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif

