#ifndef SAGA_IMPL_PACKAGES_JOB_JOB_SERVICE_HPP
#define SAGA_IMPL_PACKAGES_JOB_JOB_SERVICE_HPP

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/job_service.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif
#include <string>

#include <saga/session.hpp>
#include <impl/call.hpp>
#include <impl/engine/proxy.hpp>
#include <impl/packages/job/job_service_cpi.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/job/preprocessed/job_service.hpp>
#else

#if defined(__WAVE__)
#pragma wave option(output: "preprocessed/job_service.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {
    /**
     * This class acts as a proxy class, hiding all the nasty Saga stuff
     * (like creating and using adaptors) for the user. Users just call
     * the functions of this class like to would on every local object,
     * and don't have to worry about adaptors. It extends saga::service_cpi.
     */
    class service : public proxy
    {
      typedef v1_0::service_cpi service_cpi;
      typedef v1_0::preference_type preference_type;

      // initialize newly attached CPI instance
      void init();

     public:
      /**
       * Constructor of service, sets the resource manager of the 
       * service, and gets an instance of the Saga engine
       *
       * @param rm  the url of the resource manager to use
       * @param s   the session handle
       */
      service (saga::session const & s, std::string   const & rm);

      /*! destructor of service, destroys service and the adaptor it is 
       *  bound to */
      ~service (void);

      // API functions of service
      SAGA_CALL_IMPL_DECL_1 (saga::job::job, create_job, saga::job::description)
      SAGA_CALL_IMPL_DECL_5 (saga::job::job, run_job, std::string, std::string,
            saga::job::ostream&, saga::job::istream&, saga::job::istream&)
      SAGA_CALL_IMPL_DECL_0 (std::vector<std::string>, list)
      SAGA_CALL_IMPL_DECL_1 (saga::job::job, get_job, std::string)
      SAGA_CALL_IMPL_DECL_0 (saga::job::self, get_self)

    };

  } // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // !defined(SAGA_IMPL_PACKAGES_JOB_JOB_SERVICE_HPP)

