#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/logical_file.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(preserve: 0, output: null)
#endif

#include <saga/url.hpp>
#include <saga/adaptors/task.hpp>

#include <impl/packages/logicalfile/logical_file.hpp>
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/logical_file_cpi_instance_data.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/logicalfile/preprocessed/logical_file.cpp>
#else

#if defined(__WAVE__)
#pragma wave option(preserve: 2, output: "preprocessed/logical_file.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {
    logical_file::logical_file (saga::session const& s, std::string const& url, 
                int mode)
        : namespace_entry (saga::object::LogicalFile, s)
    {
        // initialize the instance data
        typedef adaptors::v1_0::logical_file_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> 
            logical_file_instance_data;

        {
            logical_file_instance_data data;
            data.init_data (this, TR1::shared_ptr <instance_data_type> (
                new instance_data_type (url, mode)));
        } // lock goes out of scope
    }
    
    logical_file::logical_file (attribute const& rhs_attr)
    :   saga::impl::namespace_entry(saga::object::LogicalFile),
        attribute_base(rhs_attr)
    {
    }
    
    void logical_file::init()
    {
        // initialize a CPI instance and execute required functions (such as
        // creating the file etc.)
        this->initcpi(saga::adaptors::logical_file_cpi);
    }
    
    logical_file::~logical_file (void) 
    {
        // free the instance data
        typedef adaptors::v1_0::logical_file_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> 
            logical_file_instance_data;

        logical_file_instance_data data;
        data.release_data (this);
    }

    // API methods not covered by namespace_entry
    SAGA_CALL_IMPL_IMPL_1 (logical_file, logical_file_cpi, saga::void_t, add_location, std::string)
    SAGA_CALL_IMPL_IMPL_1 (logical_file, logical_file_cpi, saga::void_t, remove_location, std::string)
    SAGA_CALL_IMPL_IMPL_2 (logical_file, logical_file_cpi, saga::void_t, update_location, std::string, std::string)
    SAGA_CALL_IMPL_IMPL_0 (logical_file, logical_file_cpi, std::vector<std::string>, list_locations)
    SAGA_CALL_IMPL_IMPL_2 (logical_file, logical_file_cpi, saga::void_t, replicate, std::string, int)

    // Generate a exact deep copy of this object
    saga::object logical_file::clone() const
    {
        return saga::logical_file::logical_file(
            new logical_file(static_cast<attribute_base const&>(*this)));
    }

  } // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif

