#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/namespace_dir.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005 Michel Zandstra (michelzandstra@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif
#include <saga/util.hpp>
#include <saga/url.hpp>

#include <saga/adaptors/task.hpp>

// needs tasks for async ops
#include <impl/call.hpp>
#include <impl/engine/task.hpp>
#include <impl/engine/sync_async.hpp>

// own header file
#include <impl/packages/namespace/namespace_dir.hpp>
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/namespace_dir_cpi_instance_data.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/namespace/preprocessed/namespace.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: "preprocessed/namespace_dir.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {
    namespace_dir::namespace_dir (saga::object::type t, 
        saga::session const & s, std::string   const & url, int mode) 
    : namespace_entry (t, s), need_to_free_instance_data(false)
    {
        // initialize the instance data
        typedef adaptors::v1_0::namespace_dir_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> namespace_dir_instance_data;

        // make sure, url is valid and directory path ends with a '/'
        saga::url dir_url(url);
        std::string pathstr (dir_url.get_path());
        std::string::size_type pos = pathstr.find_last_of(SAGA_PATH_DELIMITERS);
        if (pathstr.size() > 0 && pathstr.size()-1 != pos) 
            pathstr += "/";

        {
            namespace_dir_instance_data data;
            data.init_data (this, TR1::shared_ptr <instance_data_type> (
                new instance_data_type (url, mode)));
            need_to_free_instance_data = true;
        } // lock goes out of scope

        need_to_free_instance_data = true;
    }
    
    void namespace_dir::init()
    {
        // initialize a CPI instance and execute required functions
        this->initcpi("namespace_dir_cpi");
    }
    
    namespace_dir::namespace_dir (saga::object::type t, saga::session const & s) 
        : namespace_entry (t, s), need_to_free_instance_data(false)
    {
    }

    namespace_dir::namespace_dir (saga::object::type t) 
        : namespace_entry (t), need_to_free_instance_data(false)
    {
    }

    namespace_dir::~namespace_dir (void) 
    {
        if (need_to_free_instance_data) {
            // free the instance data
            typedef adaptors::v1_0::namespace_dir_cpi_instance_data instance_data_type;
            typedef adaptors::instance_data <instance_data_type> namespace_dir_instance_data;

            namespace_dir_instance_data data;
            data.release_data (this);
        }
    }

    // navigation/query methods
    SAGA_CALL_IMPL_IMPL_1 (namespace_dir, namespace_dir_cpi, saga::void_t,              change_dir, std::string)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, std::vector <std::string>, list,       std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, std::vector <std::string>, find,       std::string, int)
  
    SAGA_CALL_IMPL_IMPL_1 (namespace_dir, namespace_dir_cpi, std::string,               read_link,  std::string)
    SAGA_CALL_IMPL_IMPL_1 (namespace_dir, namespace_dir_cpi, bool,                      exists,     std::string)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, bool,                      is_dir,     std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, bool,                      is_entry,   std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, bool,                      is_link,    std::string, int)
  
    // Deal with entries by entry number
    SAGA_CALL_IMPL_IMPL_0 (namespace_dir, namespace_dir_cpi, unsigned int,              get_num_entries)
    SAGA_CALL_IMPL_IMPL_1 (namespace_dir, namespace_dir_cpi, std::string,               get_entry,  unsigned int)
  
    // entry management methods
    SAGA_CALL_IMPL_IMPL_3 (namespace_dir, namespace_dir_cpi, saga::void_t,              copy,       std::string, std::string, int)
    SAGA_CALL_IMPL_IMPL_3 (namespace_dir, namespace_dir_cpi, saga::void_t,              link,       std::string, std::string, int)
    SAGA_CALL_IMPL_IMPL_3 (namespace_dir, namespace_dir_cpi, saga::void_t,              move,       std::string, std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, saga::void_t,              remove,     std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, saga::void_t,              make_dir,   std::string, int)
  
    // open methods
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, saga::name_space::entry,    open,       std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_dir, namespace_dir_cpi, saga::name_space::directory,      open_dir,   std::string, int)

  } // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif

