#ifndef SAGA_IMPL_PACKAGES_DATA_NAMESPACE_NAMESPACEDIR_HPP
#define SAGA_IMPL_PACKAGES_DATA_NAMESPACE_NAMESPACEDIR_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/namespace_dir.hpp")
#endif
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005 Michel Zandstra [michelzandstra@gmail.com]
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
#include <boost/shared_ptr.hpp>

#include <string>

#include <saga/session.hpp>
#include <impl/call.hpp>
#include <impl/engine/proxy.hpp>
#include <impl/engine/task_get_bulk_adaptor.hpp>

#include <impl/packages/namespace/namespace_dir_cpi.hpp>
#include <impl/packages/namespace/namespace_entry.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/namespace/preprocessed/namespace_dir.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: "preprocessed/namespace_dir.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {
    /**
     * This class acts as a proxy class, hiding all the nasty Saga stuff
     * (like creating and using adaptors) for the user. Users just call
     * the functions of this class like to would on every local object,
     * and don't have to worry about adaptors. It extends
     * saga::name_space::directory_cpi.
     */
    class SAGA_NAMESPACE_PACKAGE_EXPORT namespace_dir 
        : public saga::impl::namespace_entry   // v1_0::namespace_dir_cpi
    {
      typedef v1_0::namespace_dir_cpi namespace_dir_cpi;
      typedef v1_0::preference_type preference_type;
      
      bool need_to_free_instance_data;

      // initialize newly attached CPI instance
      void init();

     protected:
      namespace_dir (saga::object::type t, saga::session const & s);
      explicit namespace_dir (saga::object::type t);
      
     public:
      /**
       * Constructor of namespace_dir, sets the location of the namespace_dir, and gets
       * an instance of the Saga engine
       *
       * @param loc the full path and name of the namespace_dir
       * @param s   the session handle
       */
      namespace_dir (saga::object::type t, saga::session const & s, 
                      std::string   const & loc, 
                      int                   mode);
      
      /*! destructor of namespace_dir, destroys namespace_dir and the adaptor it is 
       *  bound to */
      ~namespace_dir (void);

      // navigation/query methods
      SAGA_CALL_IMPL_DECL_1 (saga::void_t,              change_dir, std::string);
      SAGA_CALL_IMPL_DECL_2 (std::vector <std::string>, list,       std::string, int);
      SAGA_CALL_IMPL_DECL_2 (std::vector <std::string>, find,       std::string, int);

      SAGA_CALL_IMPL_DECL_1 (std::string,               read_link,  std::string);
      SAGA_CALL_IMPL_DECL_1 (bool,                      exists,     std::string);
      SAGA_CALL_IMPL_DECL_2 (bool,                      is_dir,     std::string, int);
      SAGA_CALL_IMPL_DECL_2 (bool,                      is_entry,   std::string, int);
      SAGA_CALL_IMPL_DECL_2 (bool,                      is_link,    std::string, int);

      // Deal with entries by entry number
      SAGA_CALL_IMPL_DECL_0 (unsigned int,              get_num_entries);
      SAGA_CALL_IMPL_DECL_1 (std::string,               get_entry,  unsigned int);

      // entry management methods
      SAGA_CALL_IMPL_DECL_3 (saga::void_t,              copy,       std::string, std::string, int);
      SAGA_CALL_IMPL_DECL_3 (saga::void_t,              link,       std::string, std::string, int);
      SAGA_CALL_IMPL_DECL_3 (saga::void_t,              move,       std::string, std::string, int);
      SAGA_CALL_IMPL_DECL_2 (saga::void_t,              remove,     std::string, int);
      SAGA_CALL_IMPL_DECL_2 (saga::void_t,              make_dir,   std::string, int);

      // open methods
      SAGA_CALL_IMPL_DECL_2 (saga::name_space::entry, open,       std::string, int);
      SAGA_CALL_IMPL_DECL_2 (saga::name_space::directory, open_dir,   std::string, int);

    };

  }  // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
#endif // !defined(SAGA_DEBUG)

#endif // SAGA_IMPL_PACKAGES_DATA_NAMESPACE_NAMESPACEDIR_HPP


