#ifndef SAGA_IMPL_PACKAGES_DATA_NAMESPACE_NAMESPACEDIR_CPI_HPP
#define SAGA_IMPL_PACKAGES_DATA_NAMESPACE_NAMESPACEDIR_CPI_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/namespace_dir_cpi.hpp")
#endif
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005 Michel Zandstra [michelzandstra@gmail.com]
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
#include <string>

#include <saga/util.hpp>
#include <saga/types.hpp>
#include <saga/exception.hpp>

#include <saga/file.hpp>

#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/namespace_dir_cpi_instance_data.hpp>

#include <impl/call.hpp>
#include <impl/config.hpp>
#include <impl/engine/cpi.hpp>
#include <impl/engine/register_members.hpp>

#include <impl/packages/namespace/namespace_entry_cpi.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/namespace/preprocessed/namespace_dir_cpi.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: "preprocessed/namespace_dir_cpi.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace impl { namespace v1_0 {

    ///////////////////////////////////////////////////////////////////////////
    class namespace_dir_cpi 
        : public namespace_entry_cpi
    {
    public:
        namespace_dir_cpi   (proxy * p, cpi_info const& info, 
                TR1::shared_ptr<saga::adaptor> adaptor, cpi::flags flags) 
            : namespace_entry_cpi (p, info, adaptor, flags) 
        { 
        }
         
        ~namespace_dir_cpi (void) 
        {
        }

        cpi::type get_type() const { return cpi::NSDirectory; }

        // navigation/query methods
        SAGA_CALL_CPI_DECL_VIRT_1 (namespace_dir_cpi, saga::void_t,              change_dir, std::string);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, std::vector <std::string>, list,       std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, std::vector <std::string>, find,       std::string, int);

        SAGA_CALL_CPI_DECL_VIRT_1 (namespace_dir_cpi, std::string,               read_link,  std::string);
        SAGA_CALL_CPI_DECL_VIRT_1 (namespace_dir_cpi, bool,                      exists,     std::string);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, bool,                      is_dir,     std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, bool,                      is_entry,   std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, bool,                      is_link,    std::string, int);

        // Deal with entries by entry number_cpi
        SAGA_CALL_CPI_DECL_VIRT_0 (namespace_dir_cpi, unsigned int,              get_num_entries);
        SAGA_CALL_CPI_DECL_VIRT_1 (namespace_dir_cpi, std::string,               get_entry,  unsigned int);

        // entry management methods
        SAGA_CALL_CPI_DECL_VIRT_3 (namespace_dir_cpi, saga::void_t,              copy,       std::string, std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_3 (namespace_dir_cpi, saga::void_t,              link,       std::string, std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_3 (namespace_dir_cpi, saga::void_t,              move,       std::string, std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, saga::void_t,              remove,     std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, saga::void_t,              make_dir,   std::string, int);

        // open methods
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, saga::name_space::entry,    open,       std::string, int);
        SAGA_CALL_CPI_DECL_VIRT_2 (namespace_dir_cpi, saga::name_space::directory,      open_dir,   std::string, int);

    }; // class namespace_dir_cpi

///////////////////////////////////////////////////////////////////////////////
}}} // namespace saga::impl::v1_0

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace adaptors { namespace v1_0 {

///////////////////////////////////////////////////////////////////////////////
//  register namespace_dir CPI functions
    template <typename Derived>
    inline bool
    register_namespace_dir_functions(
        std::vector<saga::impl::v1_0::cpi_info>& infos, 
        saga::impl::v1_0::cpi::maker_type maker, 
        saga::impl::v1_0::preference_type const& prefs, 
        saga::uuid const& cpi_uuid, saga::uuid const& adaptor_uuid, 
        std::string const& cpi_name)
    {
        bool retval = false;
        saga::impl::v1_0::cpi_info info(adaptors::namespace_dir_cpi, 
            cpi_name, maker, prefs, cpi_uuid, adaptor_uuid);
              
        typedef saga::impl::v1_0::namespace_dir_cpi base_cpi;
        
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, change_dir, prefs);
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, list, prefs);     
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, read_link, prefs);
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, exists, prefs);
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, get_num_entries, prefs);
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, get_entry, prefs);
        SAGA_REGISTER_MEMBER(retval, info, base_cpi, Derived, make_dir, prefs);

        // register ambiguous cpi functions        
        typedef void (Derived::*sync_is_dir_type)(bool&, std::string, int);
        typedef saga::task_type (Derived::*async_is_dir_type)(bool&, std::string, int);
        sync_is_dir_type f_sync_is_dir_type = (sync_is_dir_type)&Derived::sync_is_dir;
        async_is_dir_type f_async_is_dir_type = (async_is_dir_type) &Derived::async_is_dir;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, is_dir, prefs,
            f_sync_is_dir_type, f_async_is_dir_type);
            
        typedef void (Derived::*sync_is_entry_type)(bool&, std::string, int);
        typedef saga::task_type (Derived::*async_is_entry_type)(bool&, std::string, int);
        sync_is_entry_type f_sync_is_entry_type = (sync_is_entry_type) &Derived::sync_is_entry;
        async_is_entry_type f_async_is_entry_type = (async_is_entry_type) &Derived::async_is_entry;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, is_entry, prefs,
            f_sync_is_entry_type, f_async_is_entry_type);
            
        typedef void (Derived::*sync_is_link_type)(bool&, std::string, int);
        typedef saga::task_type (Derived::*async_is_link_type)(bool&, std::string, int);
        sync_is_link_type f_sync_is_link_type = (sync_is_link_type) &Derived::sync_is_link;
        async_is_link_type f_async_is_link_type = (async_is_link_type) &Derived::async_is_link;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, is_link, prefs,
            f_sync_is_link_type, f_async_is_link_type);
            
        typedef void (Derived::*sync_copy_type)(saga::void_t&, std::string, std::string, int);
        typedef saga::task_type (Derived::*async_copy_type)(saga::void_t&, std::string, std::string, int);
        sync_copy_type f_sync_copy_type = (sync_copy_type) &Derived::sync_copy;
        async_copy_type f_async_copy_type = (async_copy_type) &Derived::async_copy;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, copy, prefs,
            f_sync_copy_type, f_async_copy_type);

        typedef void (Derived::*sync_link_type)(saga::void_t&, std::string, std::string, int);
        typedef saga::task_type (Derived::*async_link_type)(saga::void_t&, std::string, std::string, int);
        sync_link_type f_sync_link_type = (sync_link_type) &Derived::sync_link;
        async_link_type f_async_link_type = (async_link_type) &Derived::async_link;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, link, prefs,
            f_sync_link_type, f_async_link_type);

        typedef void (Derived::*sync_move_type)(saga::void_t&, std::string, std::string, int);
        typedef saga::task_type (Derived::*async_move_type)(saga::void_t&, std::string, std::string, int);
        sync_move_type f_sync_move_type = (sync_move_type) &Derived::sync_move;
        async_move_type f_async_move_type = (async_move_type) &Derived::async_move;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, move, prefs,
            f_sync_move_type, f_async_move_type);

        typedef void (Derived::*sync_remove_type)(saga::void_t&, std::string, int);
        typedef saga::task_type (Derived::*async_remove_type)(saga::void_t&, std::string, int);
        sync_remove_type f_sync_remove_type = (sync_remove_type) &Derived::sync_remove;
        async_remove_type f_async_remove_type = (async_remove_type) &Derived::async_remove;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, remove, prefs,
            f_sync_remove_type, f_async_remove_type);
            
        typedef void (Derived::*sync_open_type)(saga::name_space::entry&, std::string, int);
        typedef saga::task_type (Derived::*async_open_type)(saga::name_space::entry&, std::string, int);
        sync_open_type f_sync_open_type = (sync_open_type) &Derived::sync_open;
        async_open_type f_async_open_type = (async_open_type) &Derived::async_open;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, open, prefs,
            f_sync_open_type, f_async_open_type);
            
        typedef void (Derived::*sync_open_dir_type)(saga::name_space::directory&, std::string, int);
        typedef saga::task_type (Derived::*async_open_dir_type)(saga::name_space::directory&, std::string, int);
        sync_open_dir_type f_sync_open_dir_type = (sync_open_dir_type) &Derived::sync_open_dir;
        async_open_dir_type f_async_open_dir_type = (async_open_dir_type) &Derived::async_open_dir;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, open_dir, prefs,
            f_sync_open_dir_type, f_async_open_dir_type);
          
        typedef void (Derived::*sync_find_type)(std::vector<std::string>&, std::string, int);
        typedef saga::task_type (Derived::*async_find_type)(std::vector<std::string>&, std::string, int);
        sync_find_type f_sync_find_type = (sync_find_type) &Derived::sync_find;
        async_find_type f_async_find_type = (async_find_type) &Derived::async_find;
        
        SAGA_REGISTER_MEMBER_EX(retval, info, base_cpi, find, prefs,
            f_sync_find_type, f_async_find_type);

        infos.push_back(info);
        return retval;    // is true if at least one function got registered
    }

    /////////////////////////////////////////////////////////////////////////////
    template <typename Derived, typename Mutex = boost::recursive_mutex>
    class namespace_dir_cpi 
        : public saga::impl::v1_0::namespace_dir_cpi
    {
    protected:
        typedef saga::impl::v1_0::namespace_dir_cpi base_type;
        typedef saga::impl::proxy proxy;
        typedef saga::impl::v1_0::cpi cpi;
        typedef saga::impl::v1_0::cpi_info cpi_info;
        typedef saga::impl::v1_0::preference_type preference_type;
        typedef Mutex mutex_type;
        
        mutex_type mtx_;     // secure access to cpi_instance_data

    public:
        namespace_dir_cpi (proxy * p, cpi_info const& info, 
              TR1::shared_ptr<saga::adaptor> adaptor, cpi::flags flags) 
            : saga::impl::v1_0::namespace_dir_cpi (p, info, adaptor, flags) 
        { 
        }
         
        ~namespace_dir_cpi (void) 
        {
        }

        ///////////////////////////////////////////////////////////////////////
        //  instance data
        //    data associated with the API object instance, i.e. visible to all 
        //    CPI object instances associated with the API object instance 
        //    regardless of the shared library (adaptor) they are implemented in.
        typedef saga::adaptors::v1_0::namespace_dir_cpi_instance_data 
            ns_dir_instance_data;
        friend class saga::adaptors::instance_data<ns_dir_instance_data>;
        typedef      saga::adaptors::instance_data<ns_dir_instance_data> 
            instance_data;

        ///////////////////////////////////////////////////////////////////////
        // generic factory function
        static cpi* cpi_maker (proxy* p, cpi_info const& info,
            saga::ini::ini const& glob_ini, saga::ini::ini const& adap_ini,
            TR1::shared_ptr<saga::adaptor> adaptor)
        {
            return new Derived (p, info, glob_ini, adap_ini, adaptor);
        }

        ///////////////////////////////////////////////////////////////////////
        // register implemented functions
        static void 
        register_cpi(std::vector<cpi_info>& infos, preference_type prefs, 
            saga::uuid adaptor_uuid, 
            std::string const& cpi_name = saga::adaptors::namespace_dir_cpi)
        {
            // register namespace_entry CPI functions
            saga::uuid cpi_uuid;
            register_namespace_entry_functions<Derived>(infos, 
                &saga::adaptors::v1_0::namespace_dir_cpi<Derived>::cpi_maker, 
                prefs, cpi_uuid, adaptor_uuid, cpi_name);

            // register namespace_dir CPI functions
            register_namespace_dir_functions<Derived>(infos,
                &saga::adaptors::v1_0::namespace_dir_cpi<Derived>::cpi_maker, 
                prefs, cpi_uuid, adaptor_uuid, cpi_name);

            // register attribute CPI functions
            register_attribute_functions<Derived>(infos, 
                &saga::adaptors::v1_0::namespace_dir_cpi<Derived>::cpi_maker, 
                prefs, cpi_uuid, adaptor_uuid, cpi_name);
        }

        ///////////////////////////////////////////////////////////////////////
        // generic up-cast of the impl pointer       
        TR1::shared_ptr<Derived> shared_from_this()
        {
            return TR1::static_pointer_cast<Derived>(
                this->base_type::shared_from_this());
        }

    }; // class namespace_dir_cpi

///////////////////////////////////////////////////////////////////////////////
}}} // namespace saga::adaptors::v1_0

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
#endif // !defined(SAGA_DEBUG)

#endif // SAGA_IMPL_PACKAGES_DATA_NAMESPACE_NAMESPACEDIR_CPI_HPP

