#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/namespace_entry.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005 Michel Zandstra (michelzandstra@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif
#include <saga/util.hpp>
#include <saga/url.hpp>

#include <saga/adaptors/task.hpp>

// needs tasks for async ops
#include <impl/call.hpp>
#include <impl/engine/task.hpp>
#include <impl/engine/sync_async.hpp>

// own header file
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/namespace_entry_cpi_instance_data.hpp>

#include <impl/packages/namespace/namespace_entry.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/namespace/preprocessed/namespace_entry.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: "preprocessed/namespace_entry.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {
    namespace_entry::namespace_entry (saga::object::type t, 
        saga::session const & s, std::string const & url, int mode) 
      : proxy (t, s)
      , need_to_free_instance_data (false)
    {
        // initialize the instance data
        typedef adaptors::v1_0::namespace_entry_cpi_instance_data 
            instance_data_type;
        typedef adaptors::instance_data <instance_data_type>       
            namespace_entry_instance_data;

        // make sure url is valid 
        saga::url ns_url(url);
        {
            namespace_entry_instance_data data;
            data.init_data (this, TR1::shared_ptr <instance_data_type> 
                            (new instance_data_type (ns_url.get_url(), mode)));
        } // lock goes out of scope

        need_to_free_instance_data = true;
    }
    
    void namespace_entry::init()
    {
        // initialize a CPI instance and execute required functions
        this->initcpi("namespace_entry_cpi");
    }
    
    namespace_entry::namespace_entry (saga::object::type t) 
    :   proxy (t, detail::get_the_session()), need_to_free_instance_data(false)
    {
        // no need to init instance data here, this is done in the derived class
    }

    namespace_entry::namespace_entry (saga::object::type t, 
            saga::session const & s) 
    :   proxy (t, s), need_to_free_instance_data(false)
    {
        // no need to init instance data here, this is done in the derived class
    }

    namespace_entry::~namespace_entry (void) 
    {
        if (need_to_free_instance_data) {
            // free the instance data
            typedef adaptors::v1_0::namespace_entry_cpi_instance_data instance_data_type;
            typedef adaptors::instance_data <instance_data_type> namespace_entry_instance_data;

            namespace_entry_instance_data data;
            data.release_data (this);
        }
    }

    // basic properties: get url
    SAGA_CALL_IMPL_IMPL_0 (namespace_entry, namespace_entry_cpi, std::string,  get_url  )
    SAGA_CALL_IMPL_IMPL_0 (namespace_entry, namespace_entry_cpi, std::string,  get_cwd  )
    SAGA_CALL_IMPL_IMPL_0 (namespace_entry, namespace_entry_cpi, std::string,  get_name )
  
    // navigation/query methods
    SAGA_CALL_IMPL_IMPL_0 (namespace_entry, namespace_entry_cpi, std::string,  read_link)
    SAGA_CALL_IMPL_IMPL_1 (namespace_entry, namespace_entry_cpi, bool,         is_dir,   int)
    SAGA_CALL_IMPL_IMPL_1 (namespace_entry, namespace_entry_cpi, bool,         is_entry, int)
    SAGA_CALL_IMPL_IMPL_1 (namespace_entry, namespace_entry_cpi, bool,         is_link,  int)
  
    // entry management methods
    SAGA_CALL_IMPL_IMPL_2 (namespace_entry, namespace_entry_cpi, saga::void_t, copy,    std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_entry, namespace_entry_cpi, saga::void_t, link,    std::string, int)
    SAGA_CALL_IMPL_IMPL_2 (namespace_entry, namespace_entry_cpi, saga::void_t, move,    std::string, int)
    SAGA_CALL_IMPL_IMPL_1 (namespace_entry, namespace_entry_cpi, saga::void_t, remove,  int)
    SAGA_CALL_IMPL_IMPL_1 (namespace_entry, namespace_entry_cpi, saga::void_t, close,   double)
  
  } // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif // SAGA_DEBUG

