#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 0, output: "preprocessed/stream.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

// needs tasks for async ops
#include <saga/util.hpp>
#include <saga/adaptors/task.hpp>

#include <impl/call.hpp>
#include <impl/engine/task.hpp>
#include <impl/engine/sync_async.hpp>

// own header file
#include <impl/packages/stream/stream.hpp>
#include <saga/adaptors/instance_data.hpp>
#include <saga/adaptors/packages/stream_cpi_instance_data.hpp>

#ifdef SAGA_DEBUG
#include <impl/packages/stream/preprocessed/stream.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: "preprocessed/stream.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{ 
  namespace impl 
  {
    stream::stream (saga::session const & s, std::string   const & loc) 
      : proxy(saga::object::Stream, s)
    {
        // initialize the instance data
        typedef adaptors::v1_0::stream_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> stream_instance_data;

        stream_instance_data data;
        data.init_data (this, 
            TR1::shared_ptr <instance_data_type>(new instance_data_type (loc)));
    }

    stream::stream (saga::adaptors::v1_0::stream_cpi_instance_data* instdata, 
            saga::session const& s)
      : proxy(saga::object::Stream, s)
    {
        // initialize the instance data
        typedef adaptors::v1_0::stream_cpi_instance_data instance_data_type;
        typedef adaptors::instance_data <instance_data_type> stream_instance_data;

        stream_instance_data data;
        data.init_data(this, TR1::shared_ptr<instance_data_type>(instdata));
    }

    void stream::init()
    {
        // initialize a CPI instance and execute required functions
        this->initcpi(saga::adaptors::stream_cpi);
    }

    stream::~stream (void) 
    {
      // release all attached adaptors now
      saga::impl::proxy::release_cpis();
        
      // free the instance data
      typedef adaptors::v1_0::stream_cpi_instance_data instance_data_type;
      typedef adaptors::instance_data <instance_data_type> stream_instance_data;

      stream_instance_data data;
      data.release_data (this);
    }

    SAGA_CALL_IMPL_IMPL_0 (stream, stream_cpi, std::string,   get_url)
    SAGA_CALL_IMPL_IMPL_0 (stream, stream_cpi, saga::context, get_context )

    SAGA_CALL_IMPL_IMPL_0 (stream, stream_cpi, saga::context,  connect     )
    SAGA_CALL_IMPL_IMPL_2 (stream, stream_cpi, std::vector<saga::stream::activity>,  wait, saga::stream::activity, double)
    SAGA_CALL_IMPL_IMPL_1 (stream, stream_cpi, saga::void_t,  close, double)
    SAGA_CALL_IMPL_IMPL_2 (stream, stream_cpi, saga::ssize_t, read, saga::mutable_buffer, saga::ssize_t)
    SAGA_CALL_IMPL_IMPL_2 (stream, stream_cpi, saga::ssize_t, write, saga::const_buffer, saga::ssize_t)

  } // namespace impl

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

#endif // SAGA_DEBUG

