//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <impl/runtime.hpp>

#include <saga/session.hpp>
#include <impl/session.hpp>

#include <saga/task_base.hpp>
#include <saga/task.hpp>

#include <saga/task_container.hpp>
#include <impl/engine/task_container.hpp>

#include <saga/url.hpp>
#include <impl/url.hpp>

/// @cond
/** These methods are not within API scope */
namespace saga { namespace impl {

    // the SAGA runtime helper class
    SAGA_EXPORT TR1::shared_ptr <impl::session> 
        runtime::get_impl (saga::session const & s)
    {
        return (s.get_impl ());
    }
    
    SAGA_EXPORT TR1::shared_ptr <impl::url> 
        runtime::get_impl (saga::url const & u)
    {
        return u.get_impl();
    }
    
    SAGA_EXPORT task_interface const*
        runtime::get_impl (task_type const & t)
    {
        return (t.get_task_if ());
    }
    
    SAGA_EXPORT task_interface*
        runtime::get_impl (task_type& t)
    {
        return (t.get_task_if ());
    }
    
    SAGA_EXPORT TR1::shared_ptr <task_container> 
        runtime::get_impl (saga::task_container const & tc)
    {
        return (tc.get_impl ());
    }

    // get the implementation of a given object
    SAGA_EXPORT TR1::shared_ptr<impl::object> 
        runtime::get_impl_object (saga::object const& obj)
    {
      return (obj.get_impl());
    }
    
    // re-create a saga::object from the impl
    SAGA_EXPORT saga::object 
        runtime::get_object(TR1::shared_ptr<saga::impl::object> impl)
    {
        return saga::object(impl);
    }
    
}}
/// @endcond

