//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_SAGA_ADAPTORS_ATTRIBUTE_CPI_WRAPPER_HPP
#define SAGA_SAGA_ADAPTORS_ATTRIBUTE_CPI_WRAPPER_HPP

#include <string>
#include <vector>

#include <saga/adaptors/attribute_cpi.hpp>
#include <impl/engine/attribute_interface.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace adaptors
{
    //  This wrapper is used in conjunction with the saga::attribute and 
    //  saga::adaptors::attribute classes when these are used from within a cpi 
    //  instance to access the very own attributes of this cpi instance 
    class attribute_cpi_wrapper 
    :   public saga::impl::attribute_interface
    {
        typedef std::vector <std::string> strvec_type;

    public:
        attribute_cpi_wrapper(saga::impl::v1_0::attribute_cpi* cpi_instance)
        :   attributes_(cpi_instance)
        {
        }
        
        task_type 
        get_attribute(std::string& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_get_attribute(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_get_attribute(ret, key);
        }
        task_type 
        set_attribute(saga::void_t& ret, std::string key, std::string val, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_set_attribute(ret, key, val), 
                    task_type(task_type::Done) : 
                attributes_->async_set_attribute(ret, key, val);
        }
        task_type 
        get_vector_attribute(strvec_type& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_get_vector_attribute(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_get_vector_attribute(ret, key);
        }
        task_type 
        set_vector_attribute(saga::void_t& ret, std::string key, strvec_type val, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_set_vector_attribute(ret, key, val), 
                    task_type(task_type::Done) : 
                attributes_->async_set_vector_attribute(ret, key, val);
        }
        task_type 
        remove_attribute(saga::void_t& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_remove_attribute(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_remove_attribute(ret, key);
        }
        task_type 
        list_attributes(strvec_type& ret, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_list_attributes(ret), 
                    task_type(task_type::Done) : 
                attributes_->async_list_attributes(ret);
        }
        task_type 
        find_attributes(strvec_type& ret, std::string pattern, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_find_attributes(ret, pattern), 
                    task_type(task_type::Done) : 
                attributes_->async_find_attributes(ret, pattern);
        }
        task_type 
        attribute_exists(bool& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_attribute_exists(ret, key), task_type(task_type::Done) : 
                attributes_->async_attribute_exists(ret, key);
        }
        task_type 
        attribute_is_readonly(bool& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_attribute_is_readonly(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_attribute_is_readonly(ret, key);
        }
        task_type 
        attribute_is_writable(bool& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_attribute_is_writable(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_attribute_is_writable(ret, key);
        }
        task_type 
        attribute_is_vector(bool& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_attribute_is_vector(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_attribute_is_vector(ret, key);
        }
        task_type 
        attribute_is_extended(bool& ret, std::string key, bool is_sync)
        {
            return is_sync ? 
                attributes_->sync_attribute_is_extended(ret, key), 
                    task_type(task_type::Done) : 
                attributes_->async_attribute_is_extended(ret, key);
        }

    private:
        saga::impl::v1_0::attribute_cpi* attributes_;
    };
    
///////////////////////////////////////////////////////////////////////////////
}}    // namespace saga::adaptors

#endif // !SAGA_SAGA_ADAPTORS_ATTRIBUTE_CPI_WRAPPER_HPP


