//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <boost/spirit/symbols.hpp>

#include <impl/config.hpp>
#include <impl/engine/url_grammar.hpp>
#include <impl/url.hpp>

#include <saga/url.hpp>
#include <impl/runtime.hpp>
#include <saga/adaptors/file_transfer_spec.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace adaptors
{
    // parse the given string for a file transfer specification and return true
    // if successful
    bool 
    parse_file_transfer_specification(std::string spec, 
        std::string& left_url, file_transfer_operator& mode, 
        std::string& right_url)
    {
        using namespace boost::spirit;
        using namespace phoenix;

        saga::url url_left, url_right;
        TR1::shared_ptr<impl::url> impl_left(impl::runtime::get_impl(url_left));
        TR1::shared_ptr<impl::url> impl_right(impl::runtime::get_impl(url_right));
        
        symbols<file_transfer_operator> ops;
        ops.add (">", copy_local_remote)
                (">>", append_local_remote)
                ("<", copy_remote_local)
                ("<<", append_remote_local)
            ;

        file_transfer_operator parsed_mode = unknown_mode;
        parse_info<> pi = parse(
                spec.c_str(),
                impl::url_grammar<impl::url>(*impl_left.get()) 
                >>  ops [var(parsed_mode) = arg1]
                >>  impl::url_grammar<impl::url>(*impl_right.get())
                >> !end_p,
                space_p
            );
            
        if (!pi.full)
            return false;
            
        left_url = url_left.get_url();
        right_url = url_right.get_url();
        mode = parsed_mode;
        return true;
    }

///////////////////////////////////////////////////////////////////////////////
}}
