//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

// System Header Files 

#include <iostream>

#include <saga/adaptors/utils/ini/ini.hpp>
#include <impl/engine/ini/ini.hpp>

// --------------------------------------------------------
//
//
//
saga::ini::section::section (shared_sec  impl_)
    : impl (impl_)
{
  // nothing
}

// --------------------------------------------------------
//
//
//
saga::ini::section::section (std::string filename)
    : impl (new impl_sec (filename))
{
  // nothing
}

// --------------------------------------------------------
//
//
//
saga::ini::section::section (const section & in)
    : impl (in.get_impl ())
{
  // nothing
};

// --------------------------------------------------------
//
//
//
saga::ini::section::section (impl_sec   * in)
    : impl (in)
{
  // nothing
}

// --------------------------------------------------------
//
//
//
saga::ini::section::~section (void) 
{
  // nothing
};

// --------------------------------------------------------
//
//
//
void saga::ini::section::read (std::string filename)
{
  impl->read (filename);
}

// --------------------------------------------------------
//
//
//
void saga::ini::section::merge (std::string second)
{
  impl->merge (second);
}

// --------------------------------------------------------
//
//
//
void saga::ini::section::merge (const saga::ini::section & second)
{
  saga::impl::ini::section * s = second.get_impl ().get ();
  impl->merge (*s);
}

// --------------------------------------------------------
//
//
//
void saga::ini::section::dump (int            ind, 
                               std::ostream & strm) const
{
  impl->dump (ind, strm);
}

// --------------------------------------------------------
//
//
//
void saga::ini::section::add_section (std::string   sec_name, 
                                const section     & sec, 
                                      section     & root)
{
  // FIXME: need to remember the root of the new section
  saga::impl::ini::section * s = sec .get_impl ().get ();
  saga::impl::ini::section * r = root.get_impl ().get ();

  impl->add_section (sec_name, *s, r);
}

// --------------------------------------------------------
//
//
//
bool saga::ini::section::has_section (std::string sec_name) const
{
  return (impl->has_section (sec_name));
}

// --------------------------------------------------------
//
//
//
bool saga::ini::section::has_section_full (std::string sec_name) const
{
  return (impl->has_section_full (sec_name));
}

// --------------------------------------------------------
//
//
//
saga::ini::section saga::ini::section::get_section (std::string sec_name) const
{
  saga::impl::ini::section * sec = impl->get_section (sec_name);
  saga::impl::ini::section * cpy = new saga::impl::ini::section (sec->clone ());

  saga::ini::section out (cpy);

  return (out);
}

// --------------------------------------------------------
//
//
//
const saga::ini::section_map saga::ini::section::get_sections (void) const
{
  saga::ini::section_map                 out;
  saga::impl::ini::section_map           map = impl->get_sections ();
  saga::impl::ini::section_map::iterator i;

  for ( i = map.begin (); i != map.end (); ++i ) 
  {
    saga::impl::ini::section * cpy = new saga::impl::ini::section (i->second.clone ());
    saga::ini::section         sec (cpy);

    out [i->first] = sec;
  }

  return (out);
}

// --------------------------------------------------------
//
//
//
void saga::ini::section::add_entry (std::string key,                 
                                    std::string val)
{
  impl->add_entry (key, val);
}

// --------------------------------------------------------
//
//
//
bool saga::ini::section::has_entry (std::string key) const
{
  return (impl->has_entry (key));
}

// --------------------------------------------------------
//
//
//
std::string saga::ini::section::get_entry (std::string key) const
{
  return (impl->get_entry (key));
}

// --------------------------------------------------------
//
//
//
const saga::ini::entry_map saga::ini::section::get_entries (void) const
{
  saga::ini::entry_map                 out;
  saga::impl::ini::entry_map           map = impl->get_entries ();
  saga::impl::ini::entry_map::iterator i;

  for ( i = map.begin (); i != map.end (); ++i ) 
  {
    out [i->first] = i->second;
  }

   return (out);
}

// --------------------------------------------------------
//
//
//
saga::ini::section saga::ini::section::get_root (void) const  
{
  return (saga::ini::section (impl->get_root ())); 
}

// --------------------------------------------------------
//
//
//
std::string saga::ini::section::get_name (void) const  
{
  return (impl->get_name ()); 
}

// --------------------------------------------------------
//
//
//
void saga::ini::section::debug (std::string msg) const
{
  std::cout << "debuggin [" 
            << impl.use_count ()
            << "] ======= " 
            << impl->get_name ()
            << "\" -- \"" 
            << msg 
            << "\""
            << std::endl;
}

