//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_SAGA_DETAIL_ATTRIBUTE_HPP
#define SAGA_SAGA_DETAIL_ATTRIBUTE_HPP

// include stl
#include <map>
#include <vector>
#include <string>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/call.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable : 4251 4231 4660)
#endif

///////////////////////////////////////////////////////////////////////////////
// for modules not belonging to the engine attributes should not be exported
#if !defined (SAGA_CREATE_PREPROCESSED_FILES) && defined (BOOST_WINDOWS) 
#  ifdef     SAGA_ENGINE_EXPORTS
#    define  SAGA_EXPORT_ATTRIBUTE   __declspec (dllexport) 
#  elif !defined (SAGA_NO_EXPORT_ATTRIBUTE)
#    define  SAGA_EXPORT_ATTRIBUTE   __declspec (dllimport) 
#  endif
#endif

#if !defined (SAGA_EXPORT_ATTRIBUTE)
#    define  SAGA_EXPORT_ATTRIBUTE   /* empty */
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace detail
{
    /// @cond
    template <typename Derived, typename Tag>
    struct SAGA_EXPORT_ATTRIBUTE attribute_priv
    {
        typedef std::vector<std::string> strvec_type;

        static task_type get_attributepriv (Derived const& this_, 
            std::string&, std::string const&);
        static task_type set_attributepriv (Derived& this_, 
            saga::void_t&, std::string const&, std::string const&);

        static task_type get_vector_attributepriv(Derived const& this_, 
            strvec_type&, std::string const&);
        static task_type set_vector_attributepriv(Derived& this_, 
            saga::void_t&, std::string const&, strvec_type const&);
        static task_type remove_attributepriv(Derived& this_, 
            saga::void_t&, std::string const&);
        
        static task_type list_attributespriv(Derived const& this_, 
            strvec_type&);
        static task_type find_attributespriv(Derived const& this_, 
            strvec_type&, std::string const&);
        static task_type attribute_existspriv(Derived const& this_, 
            bool&, std::string const&);
        static task_type attribute_is_readonlypriv(Derived const& this_, 
            bool&, std::string const&);
        static task_type attribute_is_writablepriv(Derived const& this_, 
            bool&, std::string const&);
        static task_type attribute_is_vectorpriv(Derived const& this_, 
            bool&, std::string const&);
        static task_type attribute_is_removablepriv(Derived const& this_, 
            bool&, std::string const&);
    };
    /// @endcond

    /*! \brief The %attribute interface     
     *
     *   Detailed %description starts here.
     */
    template <typename Derived>
    struct SAGA_EXPORT_ATTRIBUTE attribute 
    {
    public:
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        typedef std::vector <std::string>              strvec_type;
        
        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        typedef std::map <std::string, std::string> strmap_type;

    private:
        template <typename Tag>
        task_type get_attributepriv (std::string& r, std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::get_attributepriv(derived(), r, p); }
        
        template <typename Tag>
        task_type set_attributepriv (saga::void_t& r, std::string const& p1, std::string const& p2, Tag)
        { return attribute_priv<Derived, Tag>::set_attributepriv(derived(), r, p1, p2); }

        template <typename Tag>
        task_type get_vector_attributepriv(strvec_type& r, std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::get_vector_attributepriv(derived(), r, p); }

        template <typename Tag>
        task_type set_vector_attributepriv(saga::void_t& r, std::string const& p1, strvec_type const& p2, Tag)
        { return attribute_priv<Derived, Tag>::set_vector_attributepriv(derived(), r, p1, p2); }

        template <typename Tag>
        task_type remove_attributepriv(saga::void_t& r, std::string const& p, Tag)
        { return attribute_priv<Derived, Tag>::remove_attributepriv(derived(), r, p); }
        
        template <typename Tag>
        task_type list_attributespriv(strvec_type& r, Tag) const
        { return attribute_priv<Derived, Tag>::list_attributespriv(derived(), r); }
        
        template <typename Tag>
        task_type find_attributespriv(strvec_type& r, std::string const& p1, Tag) const
        { return attribute_priv<Derived, Tag>::find_attributespriv(derived(), r, p1); }
        
        template <typename Tag>
        task_type attribute_existspriv(bool& r, std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::attribute_existspriv(derived(), r, p); }
        
        template <typename Tag>
        task_type attribute_is_readonlypriv(bool& r, std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::attribute_is_readonlypriv(derived(), r, p); }
        
        template <typename Tag>
        task_type attribute_is_writablepriv(bool& r, std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::attribute_is_writablepriv(derived(), r, p); }
        
        template <typename Tag>
        task_type attribute_is_vectorpriv(bool& r, std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::attribute_is_vectorpriv(derived(), r, p); }
        
        template <typename Tag>
        task_type attribute_is_removablepriv(bool&r , std::string const& p, Tag) const
        { return attribute_priv<Derived, Tag>::attribute_is_removablepriv(derived(), r, p); }
        
    public:
        ///@cond
        impl::attribute_interface* get_attr (void);
        impl::attribute_interface* get_attr (void) const;
        
        impl::object *get_target_object() const;
        ///@endcond
        
    protected:
        ///@cond
        Derived& derived() 
            { return static_cast<Derived&>(*this); }
        Derived const& derived() const 
            { return static_cast<Derived const&>(*this); }

        void init (strmap_type const& scalar_ro  = strmap_type (),
                   strmap_type const& scalar_rw  = strmap_type (),
                   strmap_type const& vector_ro  = strmap_type (),
                   strmap_type const& vector_rw  = strmap_type ());
        void init (bool extensible, bool cache_only = false);

        void init_keynames(strvec_type const& keynames);
        ///@endcond

    public:        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */  
        std::string get_attribute(std::string key) const {
            std::string ret;
            get_attributepriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(std::string, get_attribute, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */  
        saga::void_t set_attribute(std::string key, std::string value) {
            saga::void_t ret;
            set_attributepriv(ret, key, value, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_0(saga::void_t, set_attribute, std::string, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */  
        strvec_type get_vector_attribute(std::string key) const {
            strvec_type ret;
            get_vector_attributepriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(strvec_type, get_vector_attribute, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */  
        saga::void_t set_vector_attribute(std::string key, strvec_type value) {
            saga::void_t ret;
            set_vector_attributepriv(ret, key, value, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_0(saga::void_t, set_vector_attribute, std::string, strvec_type)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */  
        saga::void_t remove_attribute(std::string key) {
            saga::void_t ret;
            remove_attributepriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0(saga::void_t, remove_attribute, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */   
        strvec_type list_attributes(void) const {
            strvec_type ret;
            list_attributespriv(ret, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0(strvec_type, list_attributes)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */
        strvec_type find_attributes(std::string pattern) const {
            strvec_type ret;
            find_attributespriv(ret, pattern, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(strvec_type, find_attributes, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */
        bool attribute_exists(std::string key) const {
            bool ret;
            attribute_existspriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(bool, attribute_exists, std::string)

        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */
        bool attribute_is_readonly(std::string key) const {
            bool ret;
            attribute_is_readonlypriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(bool, attribute_is_readonly, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */
        bool attribute_is_writable(std::string key) const {
            bool ret;
            attribute_is_writablepriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(bool, attribute_is_writable, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */
        bool attribute_is_vector(std::string key) const {
            bool ret;
            attribute_is_vectorpriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(bool, attribute_is_vector, std::string)
        
        /*! \brief Brief %description starts here    
         *
         *   Detailed %description starts here.
         */
        bool attribute_is_removable(std::string key) const {
            bool ret;
            attribute_is_removablepriv(ret, key, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0(bool, attribute_is_removable, std::string)
    };
    
///////////////////////////////////////////////////////////////////////////////
}}   // namespace saga::detail

#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_SAGA_DETAIL_ATTRIBUTE_HPP

