//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_EXCEPTION_HPP
#define SAGA_EXCEPTION_HPP

// include stl
#include <exception>
#include <iostream>
#include <string>

// include saga dependencies
#include <saga/base.hpp>
#include <saga/util.hpp>
#include <saga/error.hpp>
#include <saga/object.hpp>
#include <impl/runtime.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga {

  /*! \brief Brief %description starts here     
   *
   *   Detailed %description starts here.
   */
  class SAGA_EXPORT exception : public std::exception
  {
   private:
   
     std::string msg_;
     saga::error err_;
     saga::object object_;
     
   public:

      exception (saga::object obj, std::string const & m, error e = NoSuccess) 
       : err_ (e), object_(obj)
     {
        BOOST_ASSERT(err_ >= Success && err_ <= NoSuccess);
        msg_ = std::string("SAGA(") + error_names[e] + "): " + m;
         
        SAGA_VERBOSE(SAGA_VERBOSE_LEVEL_INFO) {
          if (err_ != Success)
            std::cerr << msg_ << std::endl;
        }
     }

     /*! \brief Brief %description starts here     
      *
      *   Detailed %description starts here.
      */
      exception (std::string const & m, error e = NoSuccess) : err_ (e)
     {
        BOOST_ASSERT(err_ >= Success && err_ <= NoSuccess);
        msg_ = std::string("SAGA(") + error_names[e] + "): " + m;
         
        SAGA_VERBOSE(SAGA_VERBOSE_LEVEL_INFO) {
          if (err_ != Success)
            std::cerr << msg_ << std::endl;
        }
     }

     /*! \brief Brief %description starts here     
      *
      *   Detailed %description starts here.
      */
      ~exception (void) throw () { }

     /*! \brief Brief %description starts here     
      *
      *   Detailed %description starts here.
      */
      const char * what () const throw () 
     { 
       return (msg_.c_str ());
     }

     /*! \brief Brief %description starts here     
      *
      *   Detailed %description starts here.
      */
      const char * get_message() const throw () 
     { 
       return what();
     }

     /*! \brief Brief %description starts here     
      *
      *   Detailed %description starts here.
      */
      saga::error get_error () const throw () 
     { 
       return (err_); 
     }

     /*! \brief Brief %description starts here     
      *
      *   Detailed %description starts here.
      */
      saga::object get_object () const throw () 
     { 
       return (object_); 
     }

  }; // class exception

  /////////////////////////////////////////////////////////////////////////////
  //  adaptors::exception is not in the spec, but to be used by adaptors
  /// @cond
  /** These methods are not within API scope */
  namespace adaptors {

      class exception 
          : public saga::exception 
      {
      public:
          SAGA_EXPORT exception (saga::object obj, 
                  std::string const& name, std::string const& m, 
                  error e = NoSuccess) 
              : saga::exception (obj, name + ": " + m, e)
          {
          }

          SAGA_EXPORT ~exception (void) throw () { }
      };
  
  /////////////////////////////////////////////////////////////////////////////
  }   // namespace adaptors
  /// @endcond
  
///////////////////////////////////////////////////////////////////////////////
} // namespace saga

#endif // SAGA_EXCEPTION_HPP

