#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/advert.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  job LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

//  this is needed in every file including detail/attribute_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_ATTRIBUTE

//  this is needed in every file including detail/monitorable_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_MONITORABLE

#include <saga/task.hpp>

// include file API and implementation
#include <saga/advert.hpp>
#include <impl/advert.hpp>

#include <saga/detail/attribute_impl.hpp>
#include <saga/detail/monitorable_impl.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/advert.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/advert.cpp")
#endif

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace advert 
    {
        namespace metrics
        {
            ///@cond
            saga::metrics::init_data const advert_metric_data[] = 
            {
                // advert specific metrics
                {
                    advert_modified,
                    "Metric fires on advert entry changes, it carries the key of"
                    "the modified entry.",
                    attributes::metric_mode_readonly,
                    "1", 
                    attributes::metric_type_string,
                    ""
                },
                {
                    advert_deleted,
                    "Metric fires on advert entry deletes, it carries the key of" 
                    "the deleted entry.",
                    attributes::metric_mode_readonly,
                    "1", 
                    attributes::metric_type_string,
                    ""
                },
            };
            ///@endcond
            
            ///////////////////////////////////////////////////////////////////////////
        }   // namespace saga::metrics
        
        
        entry::entry (session const& s, std::string const& url, int mode)
        :   name_space::entry (new saga::impl::advert (s, url, mode))
        {
            // initialize attribute implementation
            this->attribute_base::init (true);    // no constraints on attributes
            
            // initialize implementation object
            this->saga::object::get_impl()->init();
            
            // initialize metrics
            std::vector<saga::metric> metrics;
            for (unsigned int i = 0; 
                 i < sizeof(saga::advert::metrics::advert_metric_data)/sizeof(saga::metrics::init_data);
                 ++i)
            {
                saga::metrics::init_data const* p = &saga::advert::metrics::advert_metric_data[i];
                saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
                               p->type, p->value);
                metrics.push_back(m);
            }
            this->monitorable_base::init (metrics);
        }
        
        entry::entry (std::string const& url, int mode)
        :   name_space::entry (new saga::impl::advert (detail::get_the_session (), url, mode))
        {
            // initialize attribute implementation
            this->attribute_base::init (true);    // no constraints on attributes
            
            // initialize implementation object
            this->saga::object::get_impl()->init();
            
            // initialize metrics
            std::vector<saga::metric> metrics;
            for (unsigned int i = 0; 
                 i < sizeof(saga::advert::metrics::advert_metric_data)/sizeof(saga::metrics::init_data);
                 ++i)
            {
                saga::metrics::init_data const* p = &saga::advert::metrics::advert_metric_data[i];
                saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
                               p->type, p->value);
                metrics.push_back(m);
            }
            this->monitorable_base::init (metrics);
        }
        
        entry::entry (void) 
        {
        }
        
        entry::entry(saga::impl::advert *impl)
        : name_space::entry (impl)
        {
        }
        
        entry::entry (saga::object const& o)
        : name_space::entry(o)
        {
            if (this->get_type() != saga::object::Advert)
            {
                SAGA_THROW("Bad type conversion.", saga::BadParameter);
            }
        }
        
        entry::~entry (void)
        {
        }
        
        entry &entry::operator= (saga::object const& o)
        {
            return this->saga::name_space::entry::operator=(o), *this;
        }
        
        TR1::shared_ptr <saga::impl::advert> entry::get_impl (void) const
        { 
            // FIXME: this needs documentation
            typedef saga::object base_type;
            return TR1::static_pointer_cast <saga::impl::advert> (
                                                                  this->base_type::get_impl ());
        }
        
        // methods not covered by the namespace_entry
        SAGA_CALL_IMP_1 (entry, saga::void_t, store_object, saga::object)
        SAGA_CALL_IMP_1 (entry, saga::object, retrieve_object, saga::session)
        
        saga::task_type entry::retrieve_objectpriv (saga::object& ret, saga::task_base::Sync)
        { 
            return get_impl()->retrieve_object(ret, saga::detail::get_the_session(), true); 
        }
        saga::task_type entry::retrieve_objectpriv (saga::object& ret, saga::task_base::ASync)
        { 
            return saga::detail::run(get_impl()->retrieve_object(ret, saga::detail::get_the_session())); 
        }
        saga::task_type entry::retrieve_objectpriv (saga::object& ret, saga::task_base::Task)
        { 
            return get_impl()->retrieve_object(ret, saga::detail::get_the_session()); 
        }
        
        SAGA_CALL_IMP_1 (entry, saga::void_t, store_string, std::string)
        SAGA_CALL_IMP_0 (entry, std::string, retrieve_string)
        
    }   // namespace advert
    
    /////////////////////////////////////////////////////////////////////////////
    //  implement the attribute functions (we need to explicitly specialize 
    //  the template because the functions are not implemented inline)
    template struct detail::attribute<advert::entry>;
    
    template struct detail::attribute_priv<advert::entry, task_base::Sync>;
    template struct detail::attribute_priv<advert::entry, task_base::Async>;
    template struct detail::attribute_priv<advert::entry, task_base::Task>;
    
    ///////////////////////////////////////////////////////////////////////////////////
} // namespace saga

#endif

