#ifndef SAGA_PACKAGES_DATA_ADVERT_ADVERT_HPP
#define SAGA_PACKAGES_DATA_ADVERT_ADVERT_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/advert.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

#include <string>
#include <vector>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/types.hpp>
#include <saga/session.hpp>
#include <saga/call.hpp>
#include <saga/detail/attribute.hpp>
#include <saga/detail/monitorable.hpp>

#include <saga/packages/namespace/namespace_entry.hpp>
#include <saga/packages/advert/config.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/advert/preprocessed/advert.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/advert.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    /*! \brief The <I>%Advert Service</I> API package 
     *
     *   Detailed %description here...
     */   
    namespace advert 
    {
        namespace metrics 
        {
            /*! \brief Brief description here 
             *
             *   Detailed %description here...
             */                
            char const* const advert_modified = "advert.Modified";
            
            /*! \brief Brief description here 
             *
             *   Detailed %description here...
             */    
            char const* const advert_deleted = "advert.Deleted";
        };
        
        
        
        /*! \brief The available mode flags for %advert package %objects 
         *
         */   
        enum flags 
        {
            Unknown         = /*  -1, */  saga::name_space::Unknown, 
            None            = /*   0, */  saga::name_space::None, 
            Overwrite       = /*   1, */  saga::name_space::Overwrite, 
            Recursive       = /*   2, */  saga::name_space::Recursive, 
            Dereference     = /*   4, */  saga::name_space::Dereference, 
            Create          = /*   8, */  saga::name_space::Create, 
            Exclusive       = /*  16, */  saga::name_space::Exclusive, 
            Lock            = /*  32, */  saga::name_space::Lock, 
            CreateParents   = /*  64, */  saga::name_space::CreateParents, 
            // 256, reserved for Truncate
            // 512,  reserved for Append 
            Read            =    512, 
            Write           =   1024, 
            ReadWrite       =   Read | Write 
            // 2048, reserved for Binary
        }; 
        
        /*! \brief The %advert class represents an <b>entry</b> in a 
         *         saga::advert::directory. An %advert %directory is a meta data 
         *         %directory with a hierarchical namespace attached. This class
         *         inherits all methods from saga::name_space::namespace_entry and provides 
         *         additional methods to publish new and query existing adverts.
         *
         * Detailed %description starts here.
         */
        class SAGA_ADVERT_PACKAGE_EXPORT entry 
        : public saga::name_space::entry,
        public saga::detail::attribute<entry>
        {
        protected:
            /// @cond
            /** These types are not within API scope */
            friend class saga::impl::advert;
            
            friend struct saga::detail::attribute<entry>; 
            typedef saga::detail::attribute<entry> attribute_base;
            typedef saga::name_space::entry::monitorable_base monitorable_base;
            
            TR1::shared_ptr <saga::impl::advert> get_impl (void) const;
            explicit entry(saga::impl::advert *impl);
            /// @endcond
            
        private:
            
            SAGA_CALL_PRIV_1 (saga::void_t, store_object, saga::object)
            SAGA_CALL_PRIV_0 (saga::object, retrieve_object)
            SAGA_CALL_PRIV_1 (saga::object, retrieve_object, saga::session)
            SAGA_CALL_PRIV_1 (saga::void_t, store_string, std::string)
            SAGA_CALL_PRIV_0 (std::string, retrieve_string)
            
        public:
            /*! \brief Constructs an %advert %object that points to %url, opens 
             *         it in mode and associates it with the %session s.
             *  \exception saga::exception 
             */   
            entry (saga::session const& s, std::string const& url, int mode = Read);
            
            /*! \brief Constructs an %advert %object that points to %url and opens 
             *         it in mode. 
             *  \exception saga::exception 
             */      
            explicit entry (std::string const& url, int mode = Read);
            
            /*! \brief Constructs a copy of other.
             *  \exception saga::exception 
             */         
            explicit entry (saga::object const& other);
            
            /*! \brief Constructs a default %advert %object.
             *  \exception saga::exception 
             */   
            entry (void);
            
            /*! \brief Destroys the %advert %object and frees all allocated 
             *         resources.
             */
            ~entry (void);
            
            ///@cond 
            entry &operator= (saga::object const& object);
            ///@endcond
            
            
            /*! \brief Stores a %saga %object in this %advert.
             *  \exception saga::exception 
             *  Detailed %description starts here.
             *
             *  @param object The object to store
             */
            saga::void_t store_object (saga::object object) { 
                saga::void_t ret; 
                store_objectpriv (ret, object, saga::task_base::Sync()); 
                return ret;
            }
            SAGA_CALL_PUB_1_DEF_0  (saga::void_t, store_object, saga::object)
            
            /*! \brief Retrieves the %saga %object stored in this %advert.
             *  \exception saga::exception 
             *  Detailed %description starts here.
             *
             *  @return The object stored in this advert
             */
            saga::object retrieve_object (void) { 
                saga::object ret; 
                retrieve_objectpriv (ret, saga::task_base::Sync()); 
                return ret;
            }
            SAGA_CALL_PUB_0_DEF_0 (saga::object, retrieve_object)
            
            /*! \brief Retrieves the %saga %object stored in this %advert and 
             *         associates it with the %session s.
             *  \exception saga::exception 
             *  Detailed %description starts here.
             *
             *  @param s a %session %object
             *  @return The object stored in this advert
             */       
            saga::object retrieve_object (saga::session const& s) { 
                saga::object ret; 
                retrieve_objectpriv (ret, s, saga::task_base::Sync()); 
                return ret;
            } 
            SAGA_CALL_PUB_1_DEF_0  (saga::object, retrieve_object, saga::session)
            
            /*! \brief Stores a string in this %advert.
             *  \exception saga::exception 
             *  Detailed %description starts here.
             *
             *  @param str The string to store
             */
            saga::void_t store_string (std::string str) { 
                saga::void_t ret; 
                store_stringpriv (ret, str, saga::task_base::Sync()); 
                return ret;
            }
            SAGA_CALL_PUB_1_DEF_0  (saga::void_t, store_string, std::string)
            
            /*! \brief Retrieves the string stored in this %advert.
             *  \exception saga::exception 
             *  Detailed %description starts here.
             *
             *  @return The string stored in this advert
             */        
            std::string retrieve_string (void) { 
                std::string ret; 
                retrieve_stringpriv (ret, saga::task_base::Sync()); 
                return ret;
            }
            SAGA_CALL_PUB_0_DEF_0 (std::string, retrieve_string)
            
        }; // class advert
    }; // namespace advert
}; // namespace saga

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // !defined(SAGA_PACKAGES_DATA_ADVERT_ADVERT_HPP)

