#ifndef SAGA_DIRECTORY_HPP
#define SAGA_DIRECTORY_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/directory.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

// include stl
#include <string>

// include dependend spec sections
#include <saga/call.hpp>
#include <saga/base.hpp>
#include <saga/types.hpp>
#include <saga/object.hpp>
#include <saga/session.hpp>

#include <saga/packages/namespace/namespace_dir.hpp>
#include <saga/packages/file/file.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/directory.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/directory.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace filesystem {
    
    /*! \brief The SAGA %file package %directory API      
     *
     *   Detailed %description starts here.
     */
    class SAGA_FILE_PACKAGE_EXPORT directory 
        : public saga::name_space::directory
    {

    private:
        /// @cond
        /** These types are not within API scope */
        SAGA_CALL_PRIV_1 (saga::off_t,     get_size, std::string);
        SAGA_CALL_PRIV_1 (bool,            is_file,  std::string);

        SAGA_CALL_PRIV_2 (file,      open,     std::string, int);
        SAGA_CALL_PRIV_2 (directory, open_dir, std::string, int);
        /// @endcond

    protected:
        /// @cond
        /** These types are not within API scope */
        TR1::shared_ptr <saga::impl::directory> get_impl() const;
        /// @endcond
        
    public:
        /*! \brief Open the directory.     
         *
         *   Detailed %description starts here.
         */
        directory (session const & s, std::string const & url, 
                   int mode = ReadWrite);

        /*! \brief Open the directory.      
         *
         *   Detailed %description starts here.
         */
        explicit directory (std::string const & url, int mode = ReadWrite);
     
        /*! \brief Open the directory.      
         *
         *   Detailed %description starts here.
         */     
        explicit directory (saga::object const& o);

        /*! \brief Open the directory.      
         *
         *   Detailed %description starts here.
         */
        directory (void);

        /*! \brief Destroy the directory object.      
         *  
         *   Detailed %description starts here.
         */
        ~directory (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        directory &operator= (saga::object const& o);

        /*! \brief Returns the number of bytes in the file.
         *  \return    Number of bytes in the file.
         *  \param     url  Name of file to inspect.
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        saga::off_t get_size(std::string url) {
            saga::off_t ret;
            get_sizepriv (ret, url , saga::task_base::Sync()); 
            return ret;
        }
        /// @cond
        SAGA_CALL_PUB_1_DEF_0 (saga::off_t, get_size, std::string);
        /// @endcond

        /*! \brief Alias for is_entry in saga::ns_directory      
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */     
        bool is_file(std::string url) {
            bool ret;
            is_filepriv (ret, url , saga::task_base::Sync()); 
            return ret;
        }
        /// @cond
        SAGA_CALL_PUB_1_DEF_0 (bool, is_file,  std::string);
        /// @endcond

        /*! \brief Creates a new file instance.
         *  \param url File to be opened 
         *  \param flags: Flags definition operation modus 
         *  \return Opened file instance
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        file open(std::string url, int flags=Read) {
            file ret;
            openpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        /// @cond
        SAGA_CALL_PUB_2_DEF_1 (saga::filesystem::file, open, std::string, int, saga::filesystem::Read);
        /// @endcond

        /*! \brief Creates a directory object.     
         *  \return Opened directory instance
         *  \param url Name of directory to open 
         *  \param flags Flags definition operation modus
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        directory open_dir(std::string url, int flags=None) {
            directory ret;
            open_dirpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        /// @cond
        SAGA_CALL_PUB_2_DEF_1 (saga::filesystem::directory, open_dir, std::string, int, saga::filesystem::None);
        /// @endcond
        
    }; // directory
}} // namespace saga::file

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_DIRECTORY_HPP
