#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/file.cpp")
#endif
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  job LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif

// include file API and implementation
#include <saga/file.hpp>
#include <impl/file.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/file.cpp>
#else

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/file.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace filesystem {
  file::file (session const& s, std::string const & url, int mode)
    : entry (new saga::impl::file (s, url, mode))
  {
    this->saga::object::get_impl()->init();
  }

  file::file (std::string const & url, int mode)
    : entry (new saga::impl::file (detail::get_the_session (), url, mode))
  {
    this->saga::object::get_impl()->init();
  }

  file::file (saga::object const& o)
    : entry(o)
  {
      if (this->get_type() != saga::object::File)
      {
          SAGA_THROW("Bad type conversion.", saga::BadParameter);
      }
  }

  file::file (void) 
  {
  }
  
  file::~file (void)
  {
  }

  file &file::operator= (saga::object const& o)
  {
      return this->saga::name_space::entry::operator=(o), *this;
  }

  TR1::shared_ptr <saga::impl::file> file::get_impl (void) const
  { 
    // FIXME: this needs documentation
    typedef saga::object base_type;
    return TR1::static_pointer_cast <saga::impl::file> (
        this->base_type::get_impl ());
  }

  // methods not covered by the namespace_entry
  SAGA_CALL_IMP_0 (file, saga::off_t,   get_size)
  SAGA_CALL_IMP_2 (file, saga::ssize_t, read,  saga::mutable_buffer, saga::ssize_t)
  SAGA_CALL_IMP_2 (file, saga::ssize_t, write, saga::const_buffer, saga::ssize_t)
  SAGA_CALL_IMP_2 (file, saga::off_t,   seek,  saga::off_t, saga::filesystem::seek_mode)

  // scattered I/O
  SAGA_CALL_IMP_1 (file, saga::void_t, read_v,  std::vector<saga::filesystem::iovec>)
  SAGA_CALL_IMP_1 (file, saga::void_t, write_v, std::vector<saga::filesystem::const_iovec>)

  // pattern based I/O
  SAGA_CALL_IMP_1 (file, saga::ssize_t, size_p,  std::string)
  SAGA_CALL_IMP_2 (file, saga::ssize_t, read_p,  std::string, saga::mutable_buffer)
  SAGA_CALL_IMP_2 (file, saga::ssize_t, write_p, std::string, saga::const_buffer)

  // extended based I/O
  SAGA_CALL_IMP_0 (file, std::vector<std::string>, modes_e)
  SAGA_CALL_IMP_1 (file, saga::ssize_t, size_e,  std::string)
  SAGA_CALL_IMP_3 (file, saga::ssize_t, read_e,  std::string, std::string, saga::mutable_buffer)
  SAGA_CALL_IMP_3 (file, saga::ssize_t, write_e, std::string, std::string, saga::const_buffer)
}
} // namespace saga
///////////////////////////////////////////////////////////////////////////////////

#endif

