#ifndef SAGA_PACKAGES_DATA_FILE_FILE_HPP
#define SAGA_PACKAGES_DATA_FILE_FILE_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/file.hpp")
#endif
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

#include <string>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/types.hpp>
#include <saga/session.hpp>
#include <saga/call.hpp>

#include <saga/packages/namespace/namespace_entry.hpp>
#include <saga/packages/file/config.hpp>
#include <saga/packages/file/iovec.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/file.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/file.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga
{ 
    /*! \brief The <I>File Management</I> API package 
     *
     *   Detailed %description here...
     */   
    namespace filesystem {
    /*! \brief Brief %description starts here     
     *
     *  Details...
     */
    enum flags 
    {
        Unknown         = /*  -1, */  saga::name_space::Unknown       ,
        None            = /*   0, */  saga::name_space::None          ,
        Overwrite       = /*   1, */  saga::name_space::Overwrite     ,
        Recursive       = /*   2, */  saga::name_space::Recursive     ,
        Dereference     = /*   4, */  saga::name_space::Dereference,
        Create          = /*   8, */  saga::name_space::Create        ,
        Exclusive       = /*  16, */  saga::name_space::Exclusive     ,
        Lock            = /*  32, */  saga::name_space::Lock          ,
        CreateParents   = /*  64, */  saga::name_space::CreateParents ,
        Truncate        =    128,
        Append          =    256,
        Read            =    512,
        Write           =   1024,
        ReadWrite       =   Read | Write,
        Binary          =   2048 
    };       

    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    enum seek_mode
    {
        Start = 1,
        Current = 2,
        End = 3
    }; 
  
    /*! \brief The SAGA %file package %file API   
     *
     *   This class represents an open file descriptor for read/write 
     *   operations on a physical file. Its concept is similar to the file 
     *   descriptor returned by the open (2) call in POSIX. 
     */
    class SAGA_FILE_PACKAGE_EXPORT file 
      : public saga::name_space::entry
    {
                      
    private:
        /// @cond
        /** These types are not within API scope */
        SAGA_CALL_PRIV_0 (saga::off_t,   get_size);
        SAGA_CALL_PRIV_2 (saga::ssize_t, read,  saga::mutable_buffer, saga::ssize_t)
        SAGA_CALL_PRIV_2 (saga::ssize_t, write, saga::const_buffer, saga::ssize_t)
        SAGA_CALL_PRIV_2 (saga::off_t,   seek,  saga::off_t, seek_mode)

        // scattered I/O
        SAGA_CALL_PRIV_1 (saga::void_t, read_v,  std::vector<saga::filesystem::iovec>)
        SAGA_CALL_PRIV_1 (saga::void_t, write_v, std::vector<saga::filesystem::const_iovec>)

        // pattern based I/O
        SAGA_CALL_PRIV_1 (saga::ssize_t, size_p,  std::string)
        SAGA_CALL_PRIV_2 (saga::ssize_t, read_p,  std::string, saga::mutable_buffer)
        SAGA_CALL_PRIV_2 (saga::ssize_t, write_p, std::string, saga::const_buffer)

        // extended based I/O
        SAGA_CALL_PRIV_0 (std::vector<std::string>, modes_e)
        SAGA_CALL_PRIV_1 (saga::ssize_t, size_e,  std::string)
        SAGA_CALL_PRIV_3 (saga::ssize_t, read_e,  std::string, std::string, saga::mutable_buffer)
        SAGA_CALL_PRIV_3 (saga::ssize_t, write_e, std::string, std::string, saga::const_buffer)
        /// @endcond

    protected:     
        /// @cond
        /** These types are not within API scope */
        TR1::shared_ptr <saga::impl::file> get_impl (void) const;
        /// @endcond

    public:
        /*! \brief Creates the object.
         *
         *   Detailed %description starts here.
         */
        file (session const& s, std::string const& url, int mode = Read);
 
        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        explicit file (std::string const & url, int mode = Read);
 
        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        explicit file (saga::object const& o);
 
        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        file (void);

        /*! \brief Destroys the object.     
         *
         *   Detailed %description starts here.
         */
        ~file (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        file &operator= (saga::object const& o);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        saga::off_t get_size(void) {
            saga::off_t ret;
            get_sizepriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_0_DEF_0 (saga::off_t,   get_size);

        /*! \brief Reads up to length bytes from the file into 
         *         the buffer.     
         *  \param length Number of bytes to be read 
         *  \param buffer Buffer to read into
         *  \return Number of bytes successfully read
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        saga::ssize_t read(saga::mutable_buffer buffer, saga::ssize_t length=0) {
            saga::ssize_t ret;
            readpriv (ret, buffer, length, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::ssize_t, read,  saga::mutable_buffer, saga::ssize_t, 0)

        /*! \brief Writes up to length bytes from buffer into 
         *         the file at the current file position. 
         *  \param length Number of bytes to write
         *  \param buffer Data to write
         *  \return Number of bytes successfully written 
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        saga::ssize_t write(saga::const_buffer buffer, saga::ssize_t length=0) {
            saga::ssize_t ret;
            writepriv (ret, buffer, length, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::ssize_t, write, saga::const_buffer, saga::ssize_t, 0)

        /*! \brief Reposition the file pointer.    
         *  \param offset Offset in bytes to move pointer 
         *  \param mode Offset is relative to mode
         *  \return position of pointer after seek 
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        saga::off_t seek(saga::off_t offset, seek_mode mode) {
            saga::off_t ret;
            seekpriv (ret, offset, mode, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_0 (saga::off_t,   seek,  saga::off_t, seek_mode)

        /*! \brief Gather/scatter read     
         *  \param buffer_vec Array of buffer_vec structs defining 
         *         start (offset) and length (length) of each 
         *         individual read, buffer to read into, and integer 
         *         to store result into. 
         *  \exception saga::exception (on condition)
         *	
         *   Detailed %description starts here.
         */
        saga::void_t read_v(std::vector<saga::filesystem::iovec> buffer_vec) {
            saga::void_t ret;
            read_vpriv (ret, buffer_vec, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::void_t, read_v,  std::vector<saga::filesystem::iovec>)

        /*! \brief Gather/scatter write     
         *  \param buffer_vec Array of buffer_vec structs defining start (offset) 
         *         and length (length) of each individual write, and 
         *         buffers containing the data to write 
         *  \exception saga::exception (on condition)
         * 
         *   Detailed %description starts here.
         */
        saga::void_t write_v(std::vector<saga::filesystem::const_iovec> buffer_vec) {
            saga::void_t ret;
            write_vpriv (ret, buffer_vec, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::void_t, write_v, std::vector<saga::filesystem::const_iovec>)

       /*! \brief Determine the storage size required for a pattern I/O operation 
        *  \param pattern Pattern to determine size for 
        *  \return Size required for I/O operation with that pattern 
        *  \exception saga::exception (on condition)
        *
        *   Detailed %description starts here.
        */
        saga::ssize_t size_p(std::string pattern) {
            saga::ssize_t ret;
            size_ppriv (ret, pattern, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::ssize_t, size_p,  std::string)

       /*! \brief Pattern-based read     
        *
        * \param pattern Pattern specification for read operation 
        * \param buffer Buffer to store read bytes into 
        * \return Number of successfully read bytes
        * \exception saga::exception (on condition)
        *
        *   Detailed %description starts here.
         */
        saga::ssize_t read_p(std::string pattern, saga::mutable_buffer buffer) {
            saga::ssize_t ret;
            read_ppriv(ret, pattern, buffer, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_0 (saga::ssize_t, read_p,  std::string, saga::mutable_buffer)

        /*! \brief Pattern-based write
        *
        * \param pattern Pattern specification for read operation 
        * \param buffer Buffer to store read bytes into 
        * \return number of bytes successfully written 
        * \exception saga::exception (on condition)         
        *
        *   Detailed %description starts here.
        */
        saga::ssize_t write_p(std::string pattern, saga::const_buffer buffer) {
            saga::ssize_t ret;
            write_ppriv(ret, pattern, buffer, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_0 (saga::ssize_t, write_p, std::string, saga::const_buffer)

       /*! \brief List the exetnded modes available in this implementation, 
        *         and/or on server side    
        *
        * \param ret Pattern to determine size for 
        * \return Size required for I/O operation with that pattern 
        * \exception saga::exception (on condition)         
        *
        *   Detailed %description starts here.
        */     
        std::vector<std::string> modes_e() {
            std::vector<std::string> ret;
            modes_epriv(ret, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_0_DEF_0 (std::vector<std::string>, modes_e)

        /*! \brief determine the strorage size required for an extended 
         *         I/O operation 
         *  \exception saga::exception (on condition)  
         *  Detailed %description starts here.
         */
        saga::size_t size_e(std::string specification) {
            saga::ssize_t ret;
            size_epriv(ret, specification, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::ssize_t, size_e,  std::string)

        /*! \brief Extended read     
        * \param ext_mode Extended mode to use 
        * \param specification Specification of read operation 
        * \param buffer Buffer to store read bytes into 
        * \return Number of successfully read bytes 
        * \exception saga::exception (on condition)         
        *
        *   Detailed %description starts here.
        */
        saga::ssize_t read_e(std::string ext_mode, std::string specification,
                     saga::mutable_buffer buffer) {
            saga::ssize_t ret;
            read_epriv(ret, ext_mode, specification, buffer, saga::task_base::Sync());
            return ret;
        }
        SAGA_CALL_PUB_3_DEF_0 (saga::ssize_t, read_e,  std::string, std::string, saga::mutable_buffer)

        /*! \brief Extended write     
         *
         * \param ext_mode Extended mode to use 
         * \param specification Specification of write operation 
         * \param buffer Buffer to store read bytes into 
         * \return Number of successfully read bytes 
         * \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */     
        saga::ssize_t write_e(std::string ext_mode, std::string specification,
                     saga::const_buffer buffer) {
            saga::ssize_t ret;
            write_epriv(ret, ext_mode, specification, buffer, saga::task_base::Sync());
            return ret;
        }    
        SAGA_CALL_PUB_3_DEF_0 (saga::ssize_t, write_e, std::string, std::string, saga::const_buffer)
    }; // class file
    }; // namespace file
}; // namespace saga

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // !defined(SAGA_PACKAGES_DATA_FILE_FILE_HPP)

