//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

//  this is needed in every file including detail/attribute_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_ATTRIBUTE

#include <boost/assign/list_inserter.hpp>
#include <boost/assign/std/vector.hpp>

// include job API and implementation
#include <saga/job.hpp>
#include <saga/adaptors/metric.hpp>
#include <impl/job.hpp>

#include <saga/detail/attribute_impl.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga { namespace job {

    description::description ()
        : object (new saga::impl::description())
    {
        using namespace boost::assign;
        std::vector<std::string> valid_keys;
        valid_keys += 
              attributes::description_executable,
              attributes::description_arguments,
              attributes::description_environment,
              attributes::description_workingdirectory, 
              attributes::description_interactive,
              attributes::description_input,
              attributes::description_output,
              attributes::description_error,
              attributes::description_filetransfer,
              attributes::description_cleanup,
              attributes::description_jobstarttime,
              attributes::description_totalcputime,
              attributes::description_totalphysicalmemory,
              attributes::description_cpuarchitecture, 
              attributes::description_operatingsystemtype,
              attributes::description_candidatehosts,
              attributes::description_queue,
              attributes::description_jobcontact,
              attributes::description_spmdvariation,
              attributes::description_totalcpucount,
              attributes::description_numberofprocesses,
              attributes::description_processesperhost,
              attributes::description_threadsperprocess
        ;

      // initialize list of valid keys          
      this->init_keynames(valid_keys);

      strmap_type attributes_scalar_rw;
      insert(attributes_scalar_rw)
              (attributes::description_executable, "")
          ;

      // initialize attribute implementation
      this->init (strmap_type(), attributes_scalar_rw);
      this->init (false, true);   // cache only implementation
  }

  description::description(saga::impl::description *impl)
    : object (impl)
  {
  }

  description::~description (void)
  {
  }

  /////////////////////////////////////////////////////////////////////////////
  description::description (saga::object const& o)
    : saga::object (o)
  {
      if (this->get_type() != saga::object::JobDescription)
      {
          SAGA_THROW("Bad type conversion.", saga::BadParameter);
      }
  }

  description &description::operator= (saga::object const& o)
  {
      return saga::object::operator=(o), *this;
  }

  } // namespace job

  /////////////////////////////////////////////////////////////////////////////
  //  implement the attribute functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::attribute<job::description>;
  
  template struct detail::attribute_priv<job::description, task_base::Async>;
  template struct detail::attribute_priv<job::description, task_base::Task>;
  template struct detail::attribute_priv<job::description, task_base::Sync>;

///////////////////////////////////////////////////////////////////////////////////
} // namespace saga


