#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_directory.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(preserve: 0, output: null)
#endif

//  this is needed in every file including detail/attribute_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_ATTRIBUTE

// include directory api and implementation
#include <string>
#include <vector>

#include <saga/task.hpp>

// include the package we implement
#include <saga/call.hpp>
#include <saga/logicalfile.hpp>
#include <impl/logicalfile.hpp>

#include <saga/detail/attribute_impl.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/logicalfile/preprocessed/logical_directory.cpp>
#else

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_directory.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  /////////////////////////////////////////////////////////////////////////////
  namespace metrics
  {
    init_data const logical_directory_metric_data[] = 
    {
      // logical_directory specific metrics
      {
        saga::metrics::logical_directory_created_entry,
        "Metric fires on logical file creates, it carries the key of"
            "the new logical file.",
        attributes::metric_mode_readonly,
        "1", 
        attributes::metric_type_string,
        ""
      },
      {
        saga::metrics::logical_directory_modified_entry,
        "Metric fires on logical file changes, it carries the key of"
            "the modified logical file.",
        attributes::metric_mode_readonly,
        "1", 
        attributes::metric_type_string,
        ""
      },
      {
        saga::metrics::logical_directory_deleted_entry,
        "Metric fires on logical file deletes, it carries the key of" 
            "the deleted logical file.",
        attributes::metric_mode_readonly,
        "1", 
        attributes::metric_type_string,
        ""
      },
    };
      
  ///////////////////////////////////////////////////////////////////////////
  }   // namespace saga::metrics
  
  namespace logical_file {
  
  logical_directory::logical_directory (session const& s, std::string const& url, 
        int mode)
    : name_space::directory (new saga::impl::logical_directory (s, url, mode))
  {
      // initialize attribute implementation
      this->attribute_base::init (true);    // no constraints on attributes
      
      // initialize implementation object
      this->saga::object::get_impl()->init();

      // initialize metrics
      std::vector<saga::metric> metrics;
      for (unsigned int i = 0; 
           i < sizeof(metrics::logical_directory_metric_data)/sizeof(metrics::init_data);
           ++i)
      {
          metrics::init_data const* p = &metrics::logical_directory_metric_data[i];
          saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
              p->type, p->value);
          metrics.push_back(m);
      }
      this->monitorable_base::init (metrics);
  }

  logical_directory::logical_directory (std::string const & url, 
                        int                 mode)
    : name_space::directory (new saga::impl::logical_directory (detail::get_the_session (), url, mode))
  {
      // initialize attribute implementation
      this->attribute_base::init (true);    // no constraints on attributes
      
      // initialize implementation object
      this->saga::object::get_impl()->init();

      // initialize metrics
      std::vector<saga::metric> metrics;
      for (unsigned int i = 0; 
           i < sizeof(metrics::logical_directory_metric_data)/sizeof(metrics::init_data);
           ++i)
      {
          metrics::init_data const* p = &metrics::logical_directory_metric_data[i];
          saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
              p->type, p->value);
          metrics.push_back(m);
      }
      this->monitorable_base::init (metrics);
  }

  logical_directory::logical_directory(saga::object const& o)
    : name_space::directory(o)
  {
      if (this->get_type() != saga::object::LogicalDirectory)
      {
          SAGA_THROW("Bad type conversion.", saga::BadParameter);
      }
  }

  logical_directory::logical_directory (void)
  {
  }

  logical_directory::logical_directory(saga::impl::logical_directory *impl)
    : name_space::directory (impl)
  {
  }

  logical_directory::~logical_directory (void)
  {
  }

  logical_directory &logical_directory::operator= (saga::object const& o)
  {
      return this->saga::name_space::directory::operator=(o), *this;
  }

  TR1::shared_ptr <saga::impl::logical_directory> logical_directory::get_impl() const
  { 
    typedef saga::object base_type;
    return TR1::static_pointer_cast<saga::impl::logical_directory>(
        this->base_type::get_impl()); 
  }

  // get file info
  SAGA_CALL_IMP_1 (logical_directory, bool, is_file,  std::string)

  // overload open methods from namespace_dir
  SAGA_CALL_IMP_2 (logical_directory, saga::logical_file::logical_file, open, std::string, int)
  SAGA_CALL_IMP_2 (logical_directory, saga::logical_file::logical_directory, open_dir, std::string, int)
  SAGA_CALL_IMP_3 (logical_directory, std::vector<std::string>, find, std::string, std::vector<std::string>, int)

  }   // namespace logical_file

  /////////////////////////////////////////////////////////////////////////////
  //  implement the attribute functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::attribute<logical_file::logical_directory>;
  
  template struct detail::attribute_priv<logical_file::logical_directory, task_base::Sync>;
  template struct detail::attribute_priv<logical_file::logical_directory, task_base::Async>;
  template struct detail::attribute_priv<logical_file::logical_directory, task_base::Task>;
  
///////////////////////////////////////////////////////////////////////////////////
} // namespace saga
#endif

