#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_file.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  job LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif

//  this is needed in every file including detail/attribute_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_ATTRIBUTE

#include <saga/task.hpp>

// include file API and implementation
#include <saga/logicalfile.hpp>
#include <impl/logicalfile.hpp>

#include <saga/detail/attribute_impl.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/file/preprocessed/logical_file.cpp>
#else

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_file.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  /////////////////////////////////////////////////////////////////////////////
  namespace metrics
  {
    init_data const logical_file_metric_data[] = 
    {
      // logical_file specific metrics
      {
        saga::metrics::logical_file_modified,
        "Metric fires on logical file changes, it carries the key of"
            "the modified logical file.",
        attributes::metric_mode_readonly,
        "1", 
        attributes::metric_type_string,
        ""
      },
      {
        saga::metrics::logical_file_deleted,
        "Metric fires on logical file deletes, it carries the key of" 
            "the deleted logical file.",
        attributes::metric_mode_readonly,
        "1", 
        attributes::metric_type_string,
        ""
      },
    };
      
  ///////////////////////////////////////////////////////////////////////////
  }   // namespace saga::metrics
  
  namespace logical_file {
  
  logical_file::logical_file (session const& s, std::string const& url, int mode)
  :   entry (new saga::impl::logical_file (s, url, mode))
  {
      // initialize attribute implementation
      this->attribute_base::init (true);    // no constraints on attributes
      
      // initialize implementation object
      this->saga::object::get_impl()->init();

      // initialize metrics
      std::vector<saga::metric> metrics;
      for (unsigned int i = 0; 
           i < sizeof(metrics::logical_file_metric_data)/sizeof(metrics::init_data);
           ++i)
      {
          metrics::init_data const* p = &metrics::logical_file_metric_data[i];
          saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
              p->type, p->value);
          metrics.push_back(m);
      }
      this->monitorable_base::init (metrics);
  }

  logical_file::logical_file (std::string const & url, int mode)
  :   entry (new saga::impl::logical_file (detail::get_the_session (), url, mode))
  {
      // initialize attribute implementation
      this->attribute_base::init (true);    // no constraints on attributes
      
      // initialize implementation object
      this->saga::object::get_impl()->init();

      // initialize metrics
      std::vector<saga::metric> metrics;
      for (unsigned int i = 0; 
           i < sizeof(metrics::logical_file_metric_data)/sizeof(metrics::init_data);
           ++i)
      {
          metrics::init_data const* p = &metrics::logical_file_metric_data[i];
          saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
              p->type, p->value);
          metrics.push_back(m);
      }
      this->monitorable_base::init (metrics);
  }

  logical_file::logical_file(saga::object const& o)
    : entry(o)
  {
      if (this->get_type() != saga::object::LogicalFile)
      {
          SAGA_THROW("Bad type conversion.", saga::BadParameter);
      }
  }

  logical_file::logical_file (void) 
  {
  }
  
  logical_file::logical_file(saga::impl::logical_file *impl)
    : entry (impl)
  {
  }

  logical_file::~logical_file (void)
  {
  }

  logical_file &logical_file::operator= (saga::object const& o)
  {
      return this->saga::name_space::entry::operator=(o), *this;
  }

  TR1::shared_ptr <saga::impl::logical_file> logical_file::get_impl (void) const
  { 
    // FIXME: this needs documentation
    typedef saga::object base_type;
    return TR1::static_pointer_cast <saga::impl::logical_file> (
        this->base_type::get_impl ());
  }

  // methods not covered by the namespace_entry
  SAGA_CALL_IMP_1 (logical_file, saga::void_t, add_location, std::string)
  SAGA_CALL_IMP_1 (logical_file, saga::void_t, remove_location, std::string)
  SAGA_CALL_IMP_2 (logical_file, saga::void_t, update_location, std::string, std::string)
  SAGA_CALL_IMP_0 (logical_file, std::vector<std::string>, list_locations)
  SAGA_CALL_IMP_2 (logical_file, saga::void_t, replicate, std::string, int)

  }   // namespace logical_file
    
  /////////////////////////////////////////////////////////////////////////////
  //  implement the attribute functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::attribute<logical_file::logical_file>;
  
  template struct detail::attribute_priv<logical_file::logical_file, task_base::Sync>;
  template struct detail::attribute_priv<logical_file::logical_file, task_base::Async>;
  template struct detail::attribute_priv<logical_file::logical_file, task_base::Task>;
  
///////////////////////////////////////////////////////////////////////////////////
} // namespace saga

#endif

