#ifndef SAGA_PACKAGES_DATA_LOGICAL_FILE_LOGICAL_FILE_HPP
#define SAGA_PACKAGES_DATA_LOGICAL_FILE_LOGICAL_FILE_HPP

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_file.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__)
#pragma wave option(preserve: 0, output: null)
#endif

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma once
#endif 

#include <string>
#include <vector>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/base.hpp>
#include <saga/types.hpp>
#include <saga/session.hpp>
#include <saga/call.hpp>
#include <saga/detail/attribute.hpp>

#include <saga/packages/namespace/namespace_entry.hpp>
#include <saga/packages/logicalfile/config.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/logicalfile/preprocessed/logical_file.hpp>
#else

#if defined(__WAVE__)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/logical_file.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga
{
  /////////////////////////////////////////////////////////////////////////////
    namespace metrics {
        ///////////////////////////////////////////////////////////////////////////
        //  metric names for a advert entries
        char const* const logical_file_modified = "logical_file.Modified";
        char const* const logical_file_deleted = "logical_file.Deleted";
    }
    
    /*! \brief The <I>Replica Management</I> API package
     *
     *   Detailed %description here...
     */  
    namespace logical_file 
    {
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    enum flags 
    {
        Unknown         = /*  -1, */  saga::name_space::Unknown       ,
        None            = /*   0, */  saga::name_space::None          ,
        Overwrite       = /*   1, */  saga::name_space::Overwrite     ,
        Recursive       = /*   2, */  saga::name_space::Recursive     ,
        Dereference     = /*   4, */  saga::name_space::Dereference,
        Create          = /*   8, */  saga::name_space::Create        ,
        Exclusive       = /*  16, */  saga::name_space::Exclusive     ,
        Lock            = /*  32, */  saga::name_space::Lock          ,
        CreateParents   = /*  64, */  saga::name_space::CreateParents ,
        // 256, reserved for Truncate
        // 512,  reserved for Append 
        Read            =    512,
        Write           =   1024,
        ReadWrite       =   Read | Write
        // 2048, reserved for Binary
    };             
  
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_LOGICAL_FILE_PACKAGE_EXPORT logical_file 
        : public saga::name_space::entry,
          public saga::detail::attribute<logical_file>
    {
    protected:
        /// @cond
        /** These methods are not within API scope */
        friend class saga::impl::logical_file;
        friend struct saga::detail::attribute<logical_file>;  // needs to access get_impl()
        typedef saga::detail::attribute<logical_file> attribute_base;
        typedef saga::name_space::entry::monitorable_base monitorable_base;
        /// @endcond
            
    private:
        /// @cond
        /** These methods are not within API scope */
        SAGA_CALL_PRIV_1 (saga::void_t, add_location, std::string)
        SAGA_CALL_PRIV_1 (saga::void_t, remove_location, std::string)
        SAGA_CALL_PRIV_2 (saga::void_t, update_location, std::string, std::string)
        SAGA_CALL_PRIV_0 (std::vector<std::string>, list_locations)
        SAGA_CALL_PRIV_2 (saga::void_t, replicate, std::string, int)
        /// @endcond
     
    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::logical_file> get_impl (void) const;
        explicit logical_file(saga::impl::logical_file *impl);
        /// @endcond

    public:
        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        logical_file (session const& s, std::string const& url, int mode = Read);

        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        explicit logical_file (std::string const& url, int mode = Read);

        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        explicit logical_file (saga::object const& o);

        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        logical_file (void);

        /*! \brief Destroys the object. 
         *
         *   Detailed %description starts here.
         */
        ~logical_file (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        logical_file& operator=(saga::object const& o);

        /*! \brief Add a replica location to the replica set     
         *  \exception saga::exception (on condition)
         *  \param url Location to add to set 
         *
         *   Detailed %description starts here.
         */
        saga::void_t add_location(std::string url) {
            saga::void_t ret;
            add_locationpriv (ret, url, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::void_t, add_location, std::string)

        /*! \brief Remove a replica location from the replica set     
         *  \exception saga::exception (on condition)
         *  \param url Replica to remove from set 
         *
         *   Detailed %description starts here.
         */
        saga::void_t remove_location(std::string url) {
            saga::void_t ret;
            remove_locationpriv (ret, url, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::void_t, remove_location, std::string)

        /*! \brief Change a replica location in the replica set      
         *  \exception saga::exception (on condition)
         *  \param old_url Replica to be updated
         *  \param new_url Update for the replica
         *
         *   Detailed %description starts here.
         */
        saga::void_t update_location(std::string old_url, std::string new_url) {
            saga::void_t ret;
            update_locationpriv (ret, old_url, new_url, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_0 (saga::void_t, update_location, std::string, std::string)

        /*! \brief List the locations in the location set     
         *  \exception saga::exception (on condition)
         *  \return Array of locations in set 
         *
         *   Detailed %description starts here.
         */
        std::vector<std::string> list_locations(void) {
            std::vector<std::string> ret;
            list_locationspriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_0_DEF_0 (std::vector<std::string>, list_locations)

        /*! \brief Brief Replicate a file from any of the known replica locations 
         *               to a new location, and, on success, add the new replica 
         *               location to the set of associated replicas 
         *  \exception saga::exception (on condition)
         *  \param url Location to replicate to
         *
         *   Detailed %description starts here.
         */
        saga::void_t replicate(std::string url, int flags=saga::logical_file::None) {
            saga::void_t ret;
            replicatepriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::void_t, replicate, std::string, int, None)
    }; 

    }
}   

#if defined(__WAVE__)
#pragma wave option(output: null)
#endif

#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // !defined(SAGA_PACKAGES_DATA_LOGICAL_FILE_LOGICAL_FILE_HPP)

