#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_dir.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif

#include <string>
#include <vector>

// include the package and implementation
#include <saga/packages/namespace/namespace_dir.hpp>
#include <impl/packages/namespace/namespace_dir.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/namespace/preprocessed/namespace_dir.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_dir.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace name_space {
  directory::directory (saga::impl::namespace_dir *init)
      : entry (init)
  {
  }
  
  directory::directory (session const& s,  std::string const& url, 
          int mode, saga::object::type t)
    : entry (new saga::impl::namespace_dir (t, s, url, mode))
  {
    TR1::shared_ptr<saga::impl::object> obj (this->saga::object::get_impl());
    obj->init();
    if (t == saga::object::NSDirectory)
      obj->set_type((saga::object::type)obj->get_cpitype());
  }

  directory::directory (std::string const& url, int mode,
          saga::object::type t)
    : entry (new saga::impl::namespace_dir (t, detail::get_the_session (),
                                                        url, mode))
  {
    TR1::shared_ptr<saga::impl::object> obj (this->saga::object::get_impl());
    obj->init();
    if (t == saga::object::NSDirectory)
      obj->set_type((saga::object::type)obj->get_cpitype());
  }

  directory::directory (saga::object const& o)
    : saga::name_space::entry(o)
  {
  }
  
  directory::directory (void)
  {
  }

  directory::~directory (void)
  {
  }

  directory &directory::operator= (saga::object const& o)
  {
      return this->saga::name_space::entry::operator=(o), *this;
  }

  TR1::shared_ptr <saga::impl::namespace_dir> directory::get_impl() const
  { 
    typedef saga::object base_type;
    return TR1::static_pointer_cast<saga::impl::namespace_dir>(
        this->base_type::get_impl()); 
  }

  // navigation/query methods
  SAGA_CALL_IMP_1       (directory, saga::void_t,              change_dir, std::string)
  SAGA_CALL_CONST_IMP_2 (directory, std::vector <std::string>, list,       std::string, int)
  SAGA_CALL_CONST_IMP_2 (directory, std::vector <std::string>, find,       std::string, int)

  SAGA_CALL_CONST_IMP_1 (directory, std::string,               read_link,  std::string)
  SAGA_CALL_CONST_IMP_1 (directory, bool,                      exists,     std::string)
  SAGA_CALL_CONST_IMP_2 (directory, bool,                      is_dir,     std::string, int)
  SAGA_CALL_CONST_IMP_2 (directory, bool,                      is_entry,   std::string, int)
  SAGA_CALL_CONST_IMP_2 (directory, bool,                      is_link,    std::string, int)

  // Deal with entries by entry number
  SAGA_CALL_CONST_IMP_0 (directory, unsigned int,              get_num_entries)
  SAGA_CALL_CONST_IMP_1 (directory, std::string,               get_entry,  unsigned int)

  // entry management methods
  SAGA_CALL_IMP_3       (directory, saga::void_t,              copy,       std::string, std::string, int)
  SAGA_CALL_IMP_3       (directory, saga::void_t,              link,       std::string, std::string, int)
  SAGA_CALL_IMP_3       (directory, saga::void_t,              move,       std::string, std::string, int)
  SAGA_CALL_IMP_2       (directory, saga::void_t,              remove,     std::string, int)
  SAGA_CALL_IMP_2       (directory, saga::void_t,              make_dir,   std::string, int)

  // open methods
  SAGA_CALL_IMP_2       (directory, saga::name_space::entry,    open,       std::string, int)
  SAGA_CALL_IMP_2       (directory, saga::name_space::directory,      open_dir,   std::string, int)
}
} // namespace saga
///////////////////////////////////////////////////////////////////////////////////

#endif
