#ifndef SAGA_PACKAGES_DATA_NAMESPACEDIR_HPP
#define SAGA_PACKAGES_DATA_NAMESPACEDIR_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_dir.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
// include stl
#include <string>

// include dependent spec sections
#include <saga/call.hpp>
#include <saga/base.hpp>
#include <saga/session.hpp>
#include <saga/task.hpp>
#include <saga/permissions.hpp>

// include package config
#include <saga/packages/namespace/config.hpp>
#include <saga/packages/namespace/namespace_entry.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/description/preprocessed/namespace_dir.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_dir.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    namespace name_space {
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_NAMESPACE_PACKAGE_EXPORT directory 
        : public entry
    {
    
    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::namespace_dir> get_impl() const;
        typedef saga::name_space::entry::monitorable_base monitorable_base;
        /// @endcond
     
    private:
        // navigation/query methods
        SAGA_CALL_PRIV_1       (saga::void_t,              change_dir, std::string);
        SAGA_CALL_CONST_PRIV_2 (std::vector <std::string>, list,       std::string, int);
        SAGA_CALL_CONST_PRIV_2 (std::vector <std::string>, find,       std::string, int);

        SAGA_CALL_CONST_PRIV_1 (std::string,               read_link,  std::string);
        SAGA_CALL_CONST_PRIV_1 (bool,                      exists,     std::string);
        SAGA_CALL_CONST_PRIV_2 (bool,                      is_dir,     std::string, int);
        SAGA_CALL_CONST_PRIV_2 (bool,                      is_entry,   std::string, int);
        SAGA_CALL_CONST_PRIV_2 (bool,                      is_link,    std::string, int);

        // Deal with entries by entry number
        SAGA_CALL_CONST_PRIV_0 (unsigned int,              get_num_entries);
        SAGA_CALL_CONST_PRIV_1 (std::string,               get_entry,  unsigned int);

        // entry management methods
        SAGA_CALL_PRIV_3       (saga::void_t,              copy,       std::string, std::string, int);
        SAGA_CALL_PRIV_3       (saga::void_t,              link,       std::string, std::string, int);
        SAGA_CALL_PRIV_3       (saga::void_t,              move,       std::string, std::string, int);
        SAGA_CALL_PRIV_2       (saga::void_t,              remove,     std::string, int);
        SAGA_CALL_PRIV_2       (saga::void_t,              make_dir,   std::string, int);

        // open methods
        SAGA_CALL_PRIV_2       (saga::name_space::entry,     open,       std::string, int);
        SAGA_CALL_PRIV_2       (saga::name_space::directory, open_dir,   std::string, int);

    protected:
        /// @cond
        /** These methods are not within API scope */
        directory (saga::impl::namespace_dir *);
        /// @endcond
         
    public:
        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        directory (session const& s, std::string const& url, int mode = None,
                       saga::object::type t = saga::object::NSDirectory);

        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        explicit directory (std::string const& url, int mode = None,
            saga::object::type t = saga::object::NSDirectory);
            
        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        explicit directory (saga::object const& o);
        
        /*! \brief Creates the object.      
         *
         *   Detailed %description starts here.
         */
        directory (void);
        
        /*! \brief Destroys the object.      
         *
         *   Detailed %description starts here.
         */
        ~directory (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        directory &operator= (saga::object const& o);

        /*! \brief Change the working directory.   
         *  \param target Directory to change to 
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        saga::void_t change_dir (std::string target) { 
            saga::void_t ret; 
            change_dirpriv (ret, target, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_0 (saga::void_t, change_dir, std::string);
        
        /*! \brief List entries in this directory.
         *  \exception saga::exception (on condition)
         *  \param pattern Name or pattern to list 
         *  \return Array of names matching the pattern 
         *
         *   Detailed %description starts here.
         */
        std::vector<std::string> list (std::string pattern="*", int flags=saga::name_space::None) const { 
            std::vector <std::string> ret; 
            listpriv (ret, pattern, flags,  saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_2_DEF_2 (std::vector <std::string>, list, std::string, "*", int, saga::name_space::None);
        
        /*! \brief Find entries in the current directory and below.     
         *  \exception saga::exception (on condition)
         *  \param pattern Pattern for names of entries to be found 
         *  \param flags Flags defining the operation modus 
         *  \return Array of names matching the pattern
         *
         *   Detailed %description starts here.
         */
        std::vector<std::string> find (std::string pattern, int flags=saga::name_space::Recursive) const { 
            std::vector <std::string> ret; 
            findpriv (ret, pattern, flags,  saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_2_DEF_1 (std::vector <std::string>, find, std::string, int, saga::name_space::Recursive);

        /*! \brief Returns the name of the link target.
         *  \exception saga::exception (on condition)
         *  \param url Name to be resolved
         *  \return Resolved name 
         *   Detailed %description starts here.
         */
        std::string read_link (std::string url) const { 
            std::string ret; 
            read_linkpriv (ret, url, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0 (std::string, read_link,  std::string);
        
        /*! \brief Returns true if entry exists, false otherwise      
         *  \exception saga::exception (on condition)
         *  \param url Name to be tested for existence 
         *  \return Boolean indicating existence of name 
         *
         *   Detailed %description starts here.
         */
        bool exists (std::string url) const { 
            bool ret; 
            existspriv (ret, url, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0 (bool, exists, std::string)
        
        /*! \brief Tests name for being a directory.
         *  \exception saga::exception (on condition)
         *  \param  url Name to be tested 
         *  \param flags Flags for operation 
         *  \return boolean indicating if name is a directory 
         *
         *   Detailed %description starts here.
         */
        bool is_dir (std::string url, int flags=saga::name_space::None) const { 
            bool ret; 
            is_dirpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_2_DEF_1 (bool, is_dir, std::string, int, saga::name_space::None)
        
        /*! \brief Tests name for being a namespace_entry     
         *  \exception saga::exception (on condition)
         *  \param  url Name to be tested 
         *  \param flags Flags for operation 
         *  \return Boolean indicating if name is a non-directory entry
         * 
         *   Detailed %description starts here.
         */
        bool is_entry (std::string url, int flags=saga::name_space::None) const { 
            bool ret; 
            is_entrypriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_2_DEF_1 (bool, is_entry, std::string, int, saga::name_space::None)
        
        /*! \brief Tests name for beeing a symbolic link.
<<<<<<< .mine
			*  \exception saga::exception (on condition)
			*  \param url Name to be tested 
			*  \param flags Flags for operation 
			*  \return Boolean indicating if name is a link
			* 
			*
=======
         *  \exception saga::exception (on condition)
         *  \param  url Name to be tested 
         *  \param flags Flags for operation 
         *  \return Boolean indicating if name is a link
         * 
         *
>>>>>>> .r714
         *   Detailed %description starts here.
         */
        bool is_link (std::string url, int flags=saga::name_space::None) const { 
            bool ret; 
            is_linkpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_2_DEF_1 (bool, is_link, std::string, int, saga::name_space::None)

        // make base class functions available as well
        using entry::read_link;
        using entry::is_dir;
        using entry::is_entry;
        using entry::is_link;

        /*! \brief Gives the number of entries in the directory     
         *  \exception saga::exception (on condition)
         *  \return Number of entries in the directory
         *
         *   Detailed %description starts here.
         */
        unsigned int get_num_entries (void) const { 
            unsigned int ret; 
            get_num_entriespriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0 (unsigned int, get_num_entries);
        
        /*! \brief Gives the name of an entry in the directory based upon the enumeration defined by get_num_entries      
         *  \param entry Index of entry to get
         *  \return Name of entry at index
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        std::string get_entry (unsigned int entry) const { 
            std::string ret; 
            get_entrypriv (ret, entry, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_0 (std::string, get_entry, unsigned int);

        /*! \brief Copy the entry to another part of the namespace      
         *  \param source_url Name to copy 
         *  \param dest_url Name to copy to 
         *  \param flags Flags defining the operation modus 
         *  \exception saga::exception (on condition) 
         *
         *  Detailed %description starts here.
         */
        saga::void_t copy (std::string source_url, std::string dest_url, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            copypriv (ret, source_url, dest_url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_3_DEF_1 (saga::void_t, copy, std::string, std::string, int, saga::name_space::None);
        
        /*! \brief Create a symbolic link from the source entry to the target entry so that any reference to the target refers to the source entry      
         *  \param source_url Name to link 
         *  \param dest_url Name to link to 
         *  \param flags Flags defining the operation modus 
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        saga::void_t link (std::string source_url, std::string dest_url, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            linkpriv (ret, source_url, dest_url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_3_DEF_1 (saga::void_t, link, std::string, std::string, int, saga::name_space::None);
        
        /*! \brief Rename source to target, or move source to target if target is an directory.     
         *  \param source_url Name to move 
         *  \param dest_url Name to move to 
         *  \param flags Flags defining the operation modus 
         *  \exception saga::exception (on condition) 
         *
         *
         *   Detailed %description starts here.
         */
        saga::void_t move (std::string source_url, std::string dest_url, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            movepriv (ret, source_url, dest_url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_3_DEF_1 (saga::void_t, move, std::string, std::string, int, saga::name_space::None);
        
        /*! \brief Removes the entry     
         *  \param url Entry to be removed 
         *  \exception saga::exception (on condition) 
         *   Detailed %description starts here.
         */
        saga::void_t remove (std::string url, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            removepriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::void_t, remove, std::string, int, saga::name_space::None);

        // make base class functions available as well
        using entry::copy;
        using entry::link;
        using entry::move;
        using entry::remove;

        /*! \brief Creates a new directory      
         *  \param url Directory to create 
         *  \exception saga::exception (on condition) 
         *
         *   Detailed %description starts here.
         */
        saga::void_t make_dir (std::string url, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            make_dirpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::void_t, make_dir, std::string, int, saga::name_space::None);

        /*! \brief Creates a new namespace_entry instance      
         *  \param url Entry 
         *  \param flags Flags defining the operation modus 
         *  \return Opened entry instance 
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        saga::name_space::entry open (std::string url, int flags=saga::name_space::None) { 
            saga::name_space::entry ret; 
            openpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::name_space::entry, open, std::string, int, saga::name_space::None);
        
        /*! \brief Creates a new namespace_directory instance     
         *  \param url Directory to open 
         *  \param flags Flags defining the operation
         *  \exception saga::exception (on condition)
         *  \return Opened directory instance
         *
         *   Detailed %description starts here.
         */
        saga::name_space::directory open_dir (std::string url, int flags=saga::name_space::None) { 
            saga::name_space::directory ret; 
            open_dirpriv (ret, url, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::name_space::directory, open_dir, std::string, int, saga::name_space::None);
    };
    }; // namespace_dir

} // namespace saga

#endif // !defined(SAGA_DEBUG)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_PACKAGES_DATA_NAMESPACEDIR_HPP

