#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_entry.cpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif

//  this is needed in every file including detail/monitorable_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_MONITORABLE
#define SAGA_NO_EXPORT_PERMISSIONS

#include <string>
#include <vector>

// include the package and implementation
#include <saga/packages/namespace/namespace_entry.hpp>
#include <impl/packages/namespace/namespace_entry.hpp>

#include <saga/detail/monitorable_impl.hpp>
#include <saga/detail/permissions_impl.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/namespace/preprocessed/namespace_entry.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_entry.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace name_space {
  entry::entry (saga::impl::namespace_entry * init)
    : saga::object (init)
  {
  }
  
  entry::entry (session const & session, 
          std::string const & url, int mode, saga::object::type t)
    : saga::object (new saga::impl::namespace_entry (t, session, url, mode))
  {
    TR1::shared_ptr<saga::impl::object> obj (this->saga::object::get_impl());
    obj->init();
    if (t == saga::object::NSEntry)
      obj->set_type((saga::object::type)obj->get_cpitype());
  }
  
  entry::entry (std::string const & url, 
        int mode, saga::object::type t)
    : saga::object (new saga::impl::namespace_entry (t, detail::get_the_session (), url, mode))
  {
    TR1::shared_ptr<saga::impl::object> obj (this->saga::object::get_impl());
    obj->init();
    if (t == saga::object::NSEntry)
      obj->set_type((saga::object::type)obj->get_cpitype());
  }

  entry::entry (saga::object const& o)
    : saga::object (o)
  {
  }
  
  entry::entry ()
  {
  }

  entry::~entry (void)
  {
  }

  entry &entry::operator= (saga::object const& o)
  {
      return saga::object::operator=(o), *this;
  }

  TR1::shared_ptr <saga::impl::namespace_entry> entry::get_impl (void) const
  { 
    // FIXME: this needs documentation
    typedef saga::object base_type;
    return TR1::static_pointer_cast<saga::impl::namespace_entry>( 
        this->base_type::get_impl ());
  }

  // basic properties: get url
  SAGA_CALL_CONST_IMP_0 (entry, std::string,  get_url  )
  SAGA_CALL_CONST_IMP_0 (entry, std::string,  get_cwd  )
  SAGA_CALL_CONST_IMP_0 (entry, std::string,  get_name )

  // navigation/query methods
  SAGA_CALL_CONST_IMP_0 (entry, std::string,  read_link)
  SAGA_CALL_CONST_IMP_1 (entry, bool,         is_dir,   int)
  SAGA_CALL_CONST_IMP_1 (entry, bool,         is_entry, int)
  SAGA_CALL_CONST_IMP_1 (entry, bool,         is_link,  int)

  // entry management methods
  SAGA_CALL_IMP_2       (entry, saga::void_t, copy,    std::string, int)
  SAGA_CALL_IMP_2       (entry, saga::void_t, link,    std::string, int)
  SAGA_CALL_IMP_2       (entry, saga::void_t, move,    std::string, int)
  SAGA_CALL_IMP_1       (entry, saga::void_t, remove,  int)
  SAGA_CALL_IMP_1       (entry, saga::void_t, close,   double)
  }

  /////////////////////////////////////////////////////////////////////////////
  //  implement the monitorable functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::monitorable<name_space::entry>;

  /////////////////////////////////////////////////////////////////////////////
  //  implement the permissions functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::permissions<name_space::entry>;
} // namespace saga
///////////////////////////////////////////////////////////////////////////////////

#endif // SAGA_DEBUG

