#ifndef SAGA_PACKAGES_DATA_NAMESPACEENTRY_HPP
#define SAGA_PACKAGES_DATA_NAMESPACEENTRY_HPP

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_entry.hpp")
#endif
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif

// include stl
#include <string>

// include dependent spec sections
#include <saga/call.hpp>
#include <saga/base.hpp>
#include <saga/object.hpp>
#include <saga/session.hpp>
#include <saga/task.hpp>
#include <saga/permissions.hpp>

// include package config
#include <saga/packages/namespace/config.hpp>

#include <saga/detail/monitorable.hpp>
#include <saga/detail/permissions.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4275 4660)
#endif

#ifdef SAGA_DEBUG
#include <saga/packages/description/preprocessed/namespace_entry.hpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/namespace_entry.hpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga
{
    /*! \brief The <I>Name Space</I> API package
     *
     *   Detailed %description here...
     */   
    namespace name_space {
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    enum flags
    {
        Unknown         =  -1,
        None            =   0,
        Overwrite       =   1, 
        Recursive       =   2,
        Dereference     =   4,
        Create          =   8,
        Exclusive       =  16,
        Lock            =  32,
        CreateParents   =  64
    };
    
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */
    class SAGA_NAMESPACE_PACKAGE_EXPORT entry
        : public saga::object,
          public saga::detail::monitorable<entry>,
          public saga::detail::permissions<entry>
    {

    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::namespace_entry> get_impl (void) const;
        friend struct saga::detail::monitorable<entry>;  // needs to access get_impl()
        friend struct saga::detail::permissions<entry>;
        typedef saga::detail::monitorable<entry> monitorable_base;
        /// @endcond
          
    private:
        // basic properties: get url
        SAGA_CALL_CONST_PRIV_0 (std::string,  get_url  )
        SAGA_CALL_CONST_PRIV_0 (std::string,  get_cwd  )
        SAGA_CALL_CONST_PRIV_0 (std::string,  get_name )

        // navigation/query methods
        SAGA_CALL_CONST_PRIV_0 (std::string,  read_link);
        SAGA_CALL_CONST_PRIV_1 (bool,         is_dir,   int)
        SAGA_CALL_CONST_PRIV_1 (bool,         is_entry, int)
        SAGA_CALL_CONST_PRIV_1 (bool,         is_link,  int)

        // entry management methods 
        SAGA_CALL_PRIV_2       (saga::void_t, copy,    std::string, int)
        SAGA_CALL_PRIV_2       (saga::void_t, link,    std::string, int)
        SAGA_CALL_PRIV_2       (saga::void_t, move,    std::string, int)
        SAGA_CALL_PRIV_1       (saga::void_t, remove,  int)

        SAGA_CALL_PRIV_1       (saga::void_t, close    , double)

    protected:
        /// @cond
        /** These methods are not within API scope */
        explicit entry (saga::impl::namespace_entry * init);
        /// @endcond
     
    public:
        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        entry (session const & s, std::string const & url, 
                         int mode = None, 
                         saga::object::type t = saga::object::NSEntry);

        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        explicit entry (std::string const& url, int mode = None, 
                                  saga::object::type t = saga::object::NSEntry);

        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        explicit entry (saga::object const& o);

        /*! \brief Creates the object.     
         *
         *   Detailed %description starts here.
         */
        entry (void);

        /*! \brief Destroys the object.     
         *
         *   Detailed %description starts here.
         */
        ~entry (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */
        entry &operator= (saga::object const& o);

        /*! \brief Obtains the complete url pointing to the entry 
         *  \exception saga::exception (on condition)
         *  \return url pointing to the entry
         *
         *   Detailed %description starts here.
         */
        std::string get_url (void) const { 
            std::string ret; 
            get_urlpriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0 (std::string,  get_url  )

        /*! \brief Obtains the current working directory for the entry 
         *  \exception saga::exception (on condition)
         *  \return Current working directory
         *
         *   Detailed %description starts here.
         */
        std::string get_cwd (void) const { 
            std::string ret; 
            get_cwdpriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0 (std::string,  get_cwd  )

        /*! \brief Obtains the name part of the url     
         *  \return last part of the pathname 
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        std::string get_name (void) const { 
            std::string ret; 
            get_namepriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0 (std::string,  get_name )

        /*! \brief Returns the name of the link target. 
         *  \return Resolved name
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        std::string read_link (void) const { 
            std::string ret; 
            read_linkpriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0 (std::string,  read_link)

        /*! \brief Tests entry for being a directory.     
         *  \exception saga::exception (on condition)
         *  \param flags Flags for operation
         *  \return Boolean indicating if entry is a directory
         *
         *   Detailed %description starts here.
         */
        bool is_dir (int flags=saga::name_space::None) const{ 
            bool ret; 
            is_dirpriv (ret, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_1 (bool, is_dir, int, saga::name_space::None)

        /*! \brief Tests entry for being a namespace_entry.     
         *  \return Boolean indicating if entry is a namespace_entry 
         *  \exception saga::exception (on condition)
         *  \param flags Flags for operation
         *
         *   Detailed %description starts here.
         */
        bool is_entry  (int flags=saga::name_space::None) const{ 
            bool ret; 
            is_entrypriv (ret, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_1 (bool, is_entry, int, saga::name_space::None)

        /*! \brief Tests the entry for being a link.
        *  \exception saga::exception (on condition)
        *  \return Boolean indicating if entry is a link
        *  \param flags Flags for operation
        *
        *   Detailed %description starts here.
        */
        bool is_link (int flags=saga::name_space::None) const{ 
            bool ret; 
            is_linkpriv (ret, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_1_DEF_1 (bool, is_link, int, saga::name_space::None)

        /*! \brief Copy the entry to another part of the namespace      
         *  \param target Name to copy to 
         *  \param flags Flags defining the operation modus
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        saga::void_t copy (std::string target, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            copypriv (ret, target, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::void_t, copy, std::string, int, saga::name_space::None)

        /*! \brief Creates a symbolic link from the entry to the target entry     
         *  \exception saga::exception (on condition)
         *  \param target Name to link to
         *  \param flags Flags defining the operation modus
         *
         *   Detailed %description starts here.
         */
        saga::void_t link (std::string target, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            linkpriv (ret, target, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::void_t, link, std::string, int, saga::name_space::None)

        /*! \brief Rename source to target, or move source to target if target is an directory.    
         *  \exception saga::exception (on condition)
         *  \param target Name to move to
         *  \param flags Flags defining the operation modus
         *
         *   Detailed %description starts here.
         */
        saga::void_t move (std::string target, int flags=saga::name_space::None) { 
            saga::void_t ret; 
            movepriv (ret, target, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_2_DEF_1 (saga::void_t, move, std::string, int, saga::name_space::None)

        /*! \brief Removes this entry and closes it
         *  \exception saga::exception (on condition)
         *
         *
         *   Detailed %description starts here.
         */
        saga::void_t remove (int flags=saga::name_space::None) { 
            saga::void_t ret; 
            removepriv (ret, flags, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_1 (saga::void_t, remove, int, saga::name_space::None)

        /*! \brief Closes the object.     
         *  \exception saga::exception (on condition)
         *
         *   Detailed %description starts here.
         */
        saga::void_t close (double timeout=0.0) { 
            saga::void_t ret; 
            closepriv (ret, timeout, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_1 (saga::void_t, close, double, 0.0)
    };  // namespace_entry

    }
} 

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(output: null)
#endif
#endif // !defined(SAGA_DEBUG)

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_PACKAGES_DATA_NAMESPACEENTRY_HPP

