#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/server.cpp")
#endif
//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 0, output: null)
#endif

//  this is needed in every file including detail/monitorable_impl.hpp and not 
//  belonging to the engine
#define SAGA_NO_EXPORT_MONITORABLE
#define SAGA_NO_EXPORT_PERMISSIONS

// include the package we implement
#include <saga/context.hpp>
#include <saga/stream.hpp>
#include <impl/stream.hpp>

#include <saga/detail/monitorable_impl.hpp>
#include <saga/detail/permissions_impl.hpp>

#ifdef SAGA_DEBUG
#include <saga/packages/stream/preprocessed/server.cpp>
#else

#if defined(__WAVE__) && defined(SAGA_CREATE_PREPROCESSED_FILES)
#pragma wave option(preserve: 2, line: 1, output: "preprocessed/server.cpp")
#endif
///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
    namespace stream {
    ///////////////////////////////////////////////////////////////////////////
    namespace metrics
    {
        /// @cond
        saga::metrics::init_data const server_metric_data[] = 
        {
            {
                saga::stream::metrics::server_clientconnect,
                "Metric fires if if a client connects.",
                saga::attributes::metric_mode_readonly,
                "1", 
                saga::attributes::metric_type_trigger,
                "0"
            }
        };
        /// @endcond
    }


    server::server (session const& s, std::string const& url)
      : saga::object (new saga::impl::server (s, url))
    {
        init_metrics();
        this->saga::object::get_impl()->init();
    }

    server::server (std::string const & url)
      : saga::object (new saga::impl::server (detail::get_the_session(), url))
    {
        init_metrics();
        this->saga::object::get_impl()->init();
    }

    server::server (void)
      : saga::object (new saga::impl::server (detail::get_the_session(), ""))
    {
        init_metrics();
        this->saga::object::get_impl()->init();
    }

    void server::init_metrics()
    {
        // initialize metrics
        std::vector<saga::metric> metrics;
        for (unsigned int i = 0; 
             i < sizeof(saga::stream::metrics::server_metric_data)/sizeof(saga::metrics::init_data);
             ++i)
        {
            saga::metrics::init_data const* p = &metrics::server_metric_data[i];
            saga::metric m(*this, p->name, p->description, p->mode, p->unit, 
                p->type, p->value);
            metrics.push_back(m);
        }
        this->monitorable_base_type::init (metrics);
    }
    
    server::~server (void)
    {
    }

    TR1::shared_ptr <saga::impl::server> server::get_impl (void) const
    { 
      // FIXME: this needs documentation
      typedef saga::object base_type;
      return TR1::static_pointer_cast <saga::impl::server>(
          this->base_type::get_impl ());
    }

    ///////////////////////////////////////////////////////////////////////////
    server::server (saga::object const& o)
      : saga::object (o)
    {
        if (this->get_type() != saga::object::StreamService)
        {
            SAGA_THROW("Bad type conversion.", saga::BadParameter);
        }
    }

    server &server::operator= (saga::object const& o)
    {
        return saga::object::operator=(o), *this;
    }

    // server methods
    SAGA_CALL_IMP_1       (server, saga::stream::stream, serve,   double)
    SAGA_CALL_CONST_IMP_0 (server, std::string,  get_url)
    SAGA_CALL_IMP_1       (server, saga::void_t, close,   double)
  } // namespace saga

  /////////////////////////////////////////////////////////////////////////////
  //  implement the monitorable functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::monitorable<stream::server>;  

  /////////////////////////////////////////////////////////////////////////////
  //  implement the permissions functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::permissions<stream::server>;  

///////////////////////////////////////////////////////////////////////////////////
}
#endif // SAGA_DEBUG

