//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
//  Copyright (c) 2007 Ole Weidner (oweidner@cct.lsu.edu)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#ifndef SAGA_COMM_STREAM_STREAMSERVICE_HPP
#define SAGA_COMM_STREAM_STREAMSERVICE_HPP

// include stl
#include <string>

// include dependent spec sections
#include <saga/util.hpp>
#include <saga/call.hpp>
#include <saga/base.hpp>
#include <saga/task.hpp>
#include <saga/session.hpp>
#include <saga/detail/monitorable.hpp>
#include <saga/detail/permissions.hpp>

// include package config
#include <saga/packages/stream/config.hpp>

// need stream
#include <saga/stream.hpp>

// suppress warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(push)
#pragma warning(disable: 4251 4231 4660)
#endif

////////////////////////////////////////////////////////////////////////////////
namespace saga
{
  
    namespace stream {

    namespace metrics 
    {
        char const* const server_clientconnect = "server.ClientConnect";
    }
  
    /*! \brief Brief %description starts here     
     *
     *   Detailed %description starts here.
     */    
    class SAGA_STREAM_PACKAGE_EXPORT server 
        : public saga::object,
          public saga::detail::monitorable<server>,
          public saga::detail::permissions<server>
    {
        /// @cond
        /** These methods are not within API scope */
        friend struct saga::detail::monitorable<server>;  // needs to access get_impl()
        friend struct saga::detail::permissions<server>;
        typedef saga::detail::monitorable<server> monitorable_base_type;
        /// @endcond

    protected:
        /// @cond
        /** These methods are not within API scope */
        TR1::shared_ptr <saga::impl::server> get_impl (void) const;
        /// @endcond

    private:
        // stream server methods
        SAGA_CALL_CONST_PRIV_0 (std::string,  get_url);
        SAGA_CALL_PRIV_1       (saga::stream::stream, serve, double);
        SAGA_CALL_PRIV_1       (saga::void_t, close, double);

        void init_metrics();

    public:
        /*! \brief Creates a new stream_service object.      
         *
         *   Detailed %description starts here.
         */   
        server (session const& s, std::string const& url = "");

        /*! \brief Creates a new stream_service object.     
         *
         *   Detailed %description starts here.
         */     
        explicit server (std::string const& url);

        /*! \brief Creates a new stream_service object.     
         *
         *   Detailed %description starts here.
         */      
        server ();

        /*! \brief Creates a new stream_service object.     
         *
         *   Detailed %description starts here.
         */     
        server (saga::object const& o);

        /*! \brief Destructor for stream_service object.     
         *
         *   Detailed %description starts here.
         */       
        ~server (void);

        /*! \brief Brief %description starts here     
         *
         *   Detailed %description starts here.
         */       
        server &operator= (saga::object const& o);

        /*! \brief Get the URL to be used to connect to this server    
         *  \exception saga::exception (on condition)
         *  \return String containing the URL of the connection.
         *
         *   Detailed %description starts here.
         */     
        std::string get_url(void) const {
            std::string ret;
            get_urlpriv (ret, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_CONST_PUB_0_DEF_0 (std::string,  get_url);

        /*! \brief Wait for incoming client connections   
         *  \exception saga::exception (on condition)
         *  \param timeout Number of seconds to wait for client 
         *  \return New connected stream object 
         *
         *  Detailed %description starts here.
         */     
        saga::stream::stream serve(double timeout=0.0) {
            saga::stream::stream ret;
            servepriv (ret, timeout, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_1 (saga::stream::stream, serve, double, -1.0);

        /*! \brief Closes this stream service   
         *  \exception saga::exception (on condition)
         *
         *  Detailed %description starts here.
         */     
        saga::void_t close(double timeout=0.0) {
            saga::void_t ret;
            closepriv (ret, timeout, saga::task_base::Sync()); 
            return ret;
        }
        SAGA_CALL_PUB_1_DEF_1 (saga::void_t, close, double, 0.0);
    };
    
    }
}

// re-enable warnings about dependent classes not being exported from the dll
#if defined(BOOST_MSVC)
#pragma warning(pop)
#endif

#endif // SAGA_COMM_STREAM_STREAMSERVICE_HPP

