//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#include <boost/thread.hpp>

// include the session implementation we want to use
#include <saga/session.hpp>
#include <impl/session.hpp>
#include <saga/adaptors/manage_threads.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  // Instantiate the default session object
  //
  // Most saga objects have a session attached.  If no session is explicitly
  // specified on object creation, this default session is attached instead.
  
  /////////////////////////////////////////////////////////////////////////////
  //  Helper functions to allow thread safe initializion of the static session 
  //  object
  static session get_instance()
  {
      static session s;
      return s;
  }

  static void call_once_session()
  {
      get_instance();
  }
  ///@cond
  namespace detail {
    /////////////////////////////////////////////////////////////////////////////
    //  This is an API function
    session get_the_session (void)
    {
        boost::once_flag once_flag = BOOST_ONCE_INIT;
        boost::call_once(&call_once_session, once_flag);
        
        return get_instance();
    }
  }
  ///@endcond
  namespace adaptors
  {
//       struct managed_threads : boost::thread_group
//       {
//           ~managed_threads() 
//           {
//               join_all();
//           }
//       };
//       
//       static managed_threads& get_thread_group_helper()
//       {
//           static managed_threads threads;
//           return threads;
//       }
// 
//       static void call_once_thread_group()
//       {
//           get_thread_group_helper();
//       }
// 
//       static managed_threads& get_thread_group()
//       {
//           boost::once_flag once_flag = BOOST_ONCE_INIT;
//           boost::call_once(&call_once_thread_group, once_flag);
//           
//           return get_thread_group_helper();
//       }

      /////////////////////////////////////////////////////////////////////////
      void add_managed_thread (saga::session s, boost::thread* thrd)
      {
//           get_thread_group().add_thread(thrd);
      }
      
      void remove_managed_thread (saga::session s, boost::thread* thrd)
      {
//           get_thread_group().remove_thread(thrd);
      }
  }
  
  /////////////////////////////////////////////////////////////////////////////
  session::session (void) 
    : saga::object (new saga::impl::session())
  {
      BOOST_ASSERT(get_impl());
  }
  
  session::session (saga::object const& obj) 
    : saga::object(obj)
  {
      if (this->get_type() != saga::object::Session)
      {
          SAGA_THROW("Bad type conversion.", saga::BadParameter);
      }
  }
  
  session& session::operator=(saga::object const& obj)
  {
      saga::object::operator=(obj);
      if (this->get_type() != saga::object::Session)
      {
          SAGA_THROW("Bad type conversion.", saga::BadParameter);
      }
      return *this;
  }
  
  session::~session (void)
  {
  }
  
  TR1::shared_ptr <saga::impl::session> session::get_impl (void) const
  { 
    typedef saga::object base_type;
    return TR1::static_pointer_cast<saga::impl::session>(
        this->base_type::get_impl ());
  }

  void session::add_context (context const & c)
  {
    get_impl()->add_context (c);
  }
  
  void session::remove_context (context const & c)
  {
    get_impl()->remove_context (c);
  }
  
  std::list <context> 
  session::list_contexts (saga::context_base::type t) const
  {
    return get_impl()->list_contexts (t);
  }

  /// @cond 
  /** Doxygen generates namespace function from friend impls... */  
  bool operator== (session const & lhs,
                   session const & rhs)
  {
    if ( ! lhs.get_impl() )
    {
      // IncorrrectState
      SAGA_THROW_VERBATIM(lhs, "The lhs session is not initialized", 
          saga::IncorrectState);
    }

    if ( ! rhs.get_impl() )
    {
      // IncorrrectState
      SAGA_THROW_VERBATIM(rhs, "The rhs session is not initialized", 
          saga::IncorrectState);
    }

    return (lhs.get_impl().get () == rhs.get_impl().get ()); 
  }
  /// @endcond

} // namespace saga
///////////////////////////////////////////////////////////////////////////////

