//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Distributed under the Boost Software License, Version 1.0. (See accompanying 
//  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <utility>

#include <boost/assert.hpp>

#include <saga/task.hpp>
#include <impl/task.hpp>    // include the task implementation we want to use

#include <saga/detail/monitorable_impl.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
#if defined(SAGA_USE_V2_TASK_MODEL)
  namespace detail 
  {
#endif

  task::task (saga::task_base::state s)
    : saga::object (new saga::impl::dummy_task(s))
  {
  }

  task::task (saga::impl::object * init)
    : saga::object (init)
  {
  }

  task::task (saga::object const& o)
    : saga::object (o)
  {
  }

  task::~task (void)
  {
  }
  
  task &task::operator= (saga::object const& o)
  {
      return saga::object::operator=(o), *this;
  }

  saga::impl::task_interface* task::get_task_if (void) 
  { 
    return this->saga::object::get_impl()->get_task_interface();
  }
  saga::impl::task_interface const* task::get_task_if (void) const
  { 
    return this->saga::object::get_impl()->get_task_interface();
  }

  /// @cond 
  /** Doxygen generates namespace function from friend impls... */
  bool operator== (task const & lhs, task const & rhs)
  {
    return (lhs.get_task_if() == rhs.get_task_if() ); 
  }
  /// @endcond
  
  void task::run (void)
  {
    get_task_if ()->run ();
  }

  void task::cancel (void)
  {
    get_task_if ()->cancel ();
  }

  bool task::wait (double timeout)
  {
    bool result = get_task_if ()->wait (timeout);
    
    // we throw an error on any encountered error
    if (saga::task_base::Failed == get_state())
    {
      get_task_if ()->rethrow ();
    }
    
    return result;
  }

  task::state task::get_state (void) const
  {
    return (get_task_if ()->get_state ());
  }
  
  void task::rethrow() const
  {
    if (saga::task_base::Failed == get_state())
    {
      get_task_if()->rethrow();                         // must throw
      BOOST_ASSERT(saga::task_base::Failed != get_state());
    }
  }

  saga::object task::get_object() const
  {
    return get_task_if()->get_object();
  }
  
  /////////////////////////////////////////////////////////////////////////////
  //  implement the monitorable functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::monitorable<task>;
  
#if defined(SAGA_USE_V2_TASK_MODEL)
  }
#endif
///////////////////////////////////////////////////////////////////////////////
} // namespace saga

