//  Copyright (c) 2005-2007 Andre Merzky (andre@merzky.net)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#include <utility>

#include <saga/task_container.hpp>
#include <impl/engine/task_container.hpp>

#include <saga/detail/monitorable_impl.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga {
    
  task_container::task_container (void) 
    : saga::object (new saga::impl::task_container())
  {
  }

  task_container::~task_container(void)
  {
  }

  TR1::shared_ptr <saga::impl::task_container> task_container::get_impl (void) const
  { 
    typedef saga::object base_type;
    return TR1::static_pointer_cast<saga::impl::task_container>(
        this->base_type::get_impl ());
  }

  void task_container::add_task (task_type t)
  {
    get_impl()->add_task(t);
  }

  void  task_container::remove_task (task_type t)
  {
    get_impl()->remove_task(t);
  }

  std::vector <task_type> task_container::list_tasks(void) const
  {
    return get_impl()->list_tasks();
  }

  std::vector <task_base::state> task_container::get_states(void) const
  {
    return get_impl()->get_states();
  }

  void task_container::cancel(void)
  {
    get_impl()->cancel();
  }

  std::vector <task_type> task_container::wait (const float timeout/* = -1.0*/)
  {
    return get_impl()->wait(timeout);
  }

  void task_container::run ()
  {
    get_impl()->run();
  }

  /////////////////////////////////////////////////////////////////////////////
  //  implement the monitorable functions (we need to explicitly specialize 
  //  the template because the functions are not implemented inline)
  template struct detail::monitorable<task_container>;
  
///////////////////////////////////////////////////////////////////////////////
} // namespace saga
