//  Copyright (c) 2005-2007 Hartmut Kaiser (hartmut.kaiser@gmail.com)
// 
//  Use, modification and distribution is subject to the Boost Software
//  License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

#include <string>
#include <saga/url.hpp>
#include <impl/url.hpp>

///////////////////////////////////////////////////////////////////////////////
namespace saga 
{
  url::url (std::string const & urlstr)
      : impl (new impl::url (urlstr))
  {
  }
  
  url::url (void)
      : impl (new impl::url (""))
  {
  }
  
  url::~url (void)
  {
  }
  
  TR1::shared_ptr <saga::impl::url> url::get_impl() const
  { 
    return impl;
  }

  url url::clone (void)
  {
    url new_url;
    new_url.impl.reset (new impl::url (*impl.get ()));
    return (new_url);
  }
  
  std::string url::get_url (void) const
  {
    return impl->get_url ();
  }
  
  std::string url::get_authority (void) const
  {
    return impl->get_authority ();
  }
  
  std::string url::get_scheme (void) const
  {
    return impl->get_scheme ();
  }
  
  std::string url::get_host (void) const
  {
    return impl->get_host ();
  }
  
  int url::get_port (void) const
  {
    return impl->get_port ();
  }
  
  std::string url::get_fragment (void) const
  {
    return impl->get_fragment ();
  }
  
  std::string url::get_path (void) const
  {
    return impl->get_path ();
  }
  
  std::string url::get_query (void) const
  {
    return impl->get_query ();
  }
  
  std::string url::get_userinfo (void) const
  {
    return impl->get_userinfo ();
  }
  
  std::string url::get_username (void) const
  {
    return impl->get_username ();
  }
  
  std::string url::get_password (void) const
  {
    return impl->get_password ();
  }
  
  void url::set_scheme (std::string const & scheme)
  {
    impl->change_scheme (scheme);
  }
  
  void url::set_scheme_specific_part (std::string const & scheme_specific_part)
  {
    impl->change_scheme_specific_part (scheme_specific_part);
  }
  
  void url::set_host (std::string const & host)
  {
    impl->change_host (host);
  }
  
  void url::set_port (int port)
  {
    impl->change_port (port);
  }
  
  void url::set_fragment (std::string const & fragment)
  {
    impl->change_fragment (fragment);
  }
  
  void url::set_path (std::string const & path)
  {
    impl->change_path (path);
  }
  
  void url::set_query (std::string const & query)
  {
    impl->change_query (query);
  }
  
  void url::set_userinfo(std::string const & userinfo)
  {
    impl->change_userinfo (userinfo);
  }
  
  void url::set_username(std::string const & username)
  {
    impl->change_username (username);
  }
  
  void url::set_password(std::string const & password)
  {
    impl->change_password (password);
  }
  
} // namespace saga
///////////////////////////////////////////////////////////////////////////////

